@@ -10,6 +10,7 @@
 #include "audit-util.h"
 #include "fd-util.h"
 #include "fileio.h"
+#include "iovec-util.h"
 #include "macro.h"
 #include "parse-util.h"
 #include "process-util.h"
@@ -85,10 +86,7 @@ static int try_audit_request(int fd) {
                 .hdr.nlmsg_type = AUDIT_GET_FEATURE,
                 .hdr.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK,
         };
-        iov = (struct iovec) {
-                .iov_base = &msg,
-                .iov_len = msg.hdr.nlmsg_len,
-        };
+        iov = IOVEC_MAKE(&msg, msg.hdr.nlmsg_len);
         mh = (struct msghdr) {
                 .msg_iov = &iov,
                 .msg_iovlen = 1,
@@ -101,7 +99,7 @@ static int try_audit_request(int fd) {
 
         n = recvmsg_safe(fd, &mh, 0);
         if (n < 0)
-                return n;
+                return -errno;
         if (n != NLMSG_LENGTH(sizeof(struct nlmsgerr)))
                 return -EIO;
 
