@@ -58,7 +58,7 @@
 #define UDF_SUPER_MAGIC 0x15013346
 #endif
 
-/* b1123ea6d3b3da25af5c8a9d843bd07ab63213f4 (4.8)*/
+/* b1123ea6d3b3da25af5c8a9d843bd07ab63213f4 (4.8) */
 #ifndef BALLOON_KVM_MAGIC
 #define BALLOON_KVM_MAGIC 0x13661366
 #endif
@@ -128,11 +128,6 @@
 #define DEVMEM_MAGIC 0x454d444d
 #endif
 
-/* cb12fd8e0dabb9a1c8aef55a6a41e2c255fcdf4b (6.8) */
-#ifndef PID_FS_MAGIC
-#define PID_FS_MAGIC 0x50494446
-#endif
-
 /* Not in mainline but included in Ubuntu */
 #ifndef SHIFTFS_MAGIC
 #define SHIFTFS_MAGIC 0x6a656a62
