@@ -22,7 +22,6 @@
 
 #include "macro.h"
 #include "missing_keyctl.h"
-#include "missing_sched.h"
 #include "missing_stat.h"
 #include "missing_syscall_def.h"
 
@@ -33,6 +32,21 @@
 
 /* ======================================================================= */
 
+#if !HAVE_FCHMODAT2
+static inline int missing_fchmodat2(int dirfd, const char *path, mode_t mode, int flags) {
+#  ifdef __NR_fchmodat2
+        return syscall(__NR_fchmodat2, dirfd, path, mode, flags);
+#  else
+        errno = ENOSYS;
+        return -1;
+#  endif
+}
+
+#  define fchmodat2 missing_fchmodat2
+#endif
+
+/* ======================================================================= */
+
 #if !HAVE_PIVOT_ROOT
 static inline int missing_pivot_root(const char *new_root, const char *put_old) {
         return syscall(__NR_pivot_root, new_root, put_old);
@@ -531,6 +545,10 @@ static inline int missing_open_tree(
 
 /* ======================================================================= */
 
+#ifndef MOVE_MOUNT_BENEATH
+#define MOVE_MOUNT_BENEATH 0x00000200
+#endif
+
 #if !HAVE_MOVE_MOUNT
 
 #ifndef MOVE_MOUNT_F_EMPTY_PATH
@@ -649,22 +667,6 @@ static inline ssize_t missing_getdents64(int fd, void *buffer, size_t length) {
 
 /* ======================================================================= */
 
-#if !HAVE_SCHED_SETATTR
-
-static inline ssize_t missing_sched_setattr(pid_t pid, struct sched_attr *attr, unsigned int flags) {
-#  if defined __NR_sched_setattr
-        return syscall(__NR_sched_setattr, pid, attr, flags);
-#  else
-        errno = ENOSYS;
-        return -1;
-#  endif
-}
-
-#  define sched_setattr missing_sched_setattr
-#endif
-
-/* ======================================================================= */
-
 /* glibc does not provide clone() on ia64, only clone2(). Not only that, but it also doesn't provide a
  * prototype, only the symbol in the shared library (it provides a prototype for clone(), but not the
  * symbol in the shared library). */
