@@ -49,6 +49,7 @@
 
 /* Underlined */
 #define ANSI_GREY_UNDERLINE              "\x1B[0;4;38;5;245m"
+#define ANSI_BRIGHT_BLACK_UNDERLINE      "\x1B[0;4;90m"
 #define ANSI_HIGHLIGHT_RED_UNDERLINE     "\x1B[0;1;4;31m"
 #define ANSI_HIGHLIGHT_GREEN_UNDERLINE   "\x1B[0;1;4;32m"
 #define ANSI_HIGHLIGHT_YELLOW_UNDERLINE  "\x1B[0;1;4;38;5;185m"
@@ -62,8 +63,13 @@
 #define ANSI_HIGHLIGHT_UNDERLINE "\x1B[0;1;4m"
 
 /* Fallback colors: 256 -> 16 */
-#define ANSI_HIGHLIGHT_GREY_FALLBACK   "\x1B[0;1;90m"
-#define ANSI_HIGHLIGHT_YELLOW_FALLBACK "\x1B[0;1;33m"
+#define ANSI_HIGHLIGHT_GREY_FALLBACK             "\x1B[0;1;90m"
+#define ANSI_HIGHLIGHT_GREY_FALLBACK_UNDERLINE   "\x1B[0;1;4;90m"
+#define ANSI_HIGHLIGHT_YELLOW_FALLBACK           "\x1B[0;1;33m"
+#define ANSI_HIGHLIGHT_YELLOW_FALLBACK_UNDERLINE "\x1B[0;1;4;33m"
+
+/* Background colors */
+#define ANSI_BACKGROUND_BLUE "\x1B[44m"
 
 /* Reset/clear ANSI styles */
 #define ANSI_NORMAL "\x1B[0m"
@@ -79,6 +85,7 @@
 
 int reset_terminal_fd(int fd, bool switch_to_text);
 int reset_terminal(const char *name);
+int set_terminal_cursor_position(int fd, unsigned int row, unsigned int column);
 
 int open_terminal(const char *name, int mode);
 
@@ -124,6 +131,7 @@ int terminal_vhangup_fd(int fd);
 int terminal_vhangup(const char *name);
 
 int terminal_set_size_fd(int fd, const char *ident, unsigned rows, unsigned cols);
+int proc_cmdline_tty_size(const char *tty, unsigned *ret_rows, unsigned *ret_cols);
 
 int chvt(int vt);
 
@@ -177,20 +185,24 @@ static inline bool colors_enabled(void) {
                 }                                              \
         }
 
-#define DEFINE_ANSI_FUNC_UNDERLINE(name, NAME)                            \
-        static inline const char *ansi_##name(void) {                     \
-                return underline_enabled() ? ANSI_##NAME ANSI_UNDERLINE : \
-                        colors_enabled() ? ANSI_##NAME : "";              \
+static inline const char *ansi_underline(void) {
+        return underline_enabled() ? ANSI_UNDERLINE : ANSI_NORMAL;
+}
+
+#define DEFINE_ANSI_FUNC_UNDERLINE(name, NAME)                          \
+        static inline const char *ansi_##name(void) {                   \
+                return underline_enabled() ? ANSI_##NAME##_UNDERLINE :  \
+                        colors_enabled() ? ANSI_##NAME : "";            \
         }
 
 
-#define DEFINE_ANSI_FUNC_UNDERLINE_256(name, NAME, FALLBACK)                                                           \
-        static inline const char *ansi_##name(void) {                                                                  \
-                switch (get_color_mode()) {                                                                            \
-                        case COLOR_OFF: return "";                                                                     \
-                        case COLOR_16: return underline_enabled() ? ANSI_##FALLBACK ANSI_UNDERLINE : ANSI_##FALLBACK;  \
-                        default : return underline_enabled() ? ANSI_##NAME ANSI_UNDERLINE: ANSI_##NAME;                \
-                }                                                                                                      \
+#define DEFINE_ANSI_FUNC_UNDERLINE_256(name, NAME, FALLBACK)                                                        \
+        static inline const char *ansi_##name(void) {                                                               \
+                switch (get_color_mode()) {                                                                         \
+                        case COLOR_OFF: return "";                                                                  \
+                        case COLOR_16: return underline_enabled() ? ANSI_##FALLBACK##_UNDERLINE : ANSI_##FALLBACK;  \
+                        default : return underline_enabled() ? ANSI_##NAME##_UNDERLINE: ANSI_##NAME;                \
+                }                                                                                                   \
         }
 
 DEFINE_ANSI_FUNC(normal,            NORMAL);
@@ -229,7 +241,6 @@ static inline const char* _ansi_highlight_yellow(void) {
         return colors_enabled() ? _ANSI_HIGHLIGHT_YELLOW : "";
 }
 
-DEFINE_ANSI_FUNC_UNDERLINE(underline,                       NORMAL);
 DEFINE_ANSI_FUNC_UNDERLINE(highlight_underline,             HIGHLIGHT);
 DEFINE_ANSI_FUNC_UNDERLINE_256(grey_underline,              GREY, BRIGHT_BLACK);
 DEFINE_ANSI_FUNC_UNDERLINE(highlight_red_underline,         HIGHLIGHT_RED);
