@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
-#include <efi.h>
-#include <stddef.h>
+#include "efi.h"
 
 EFI_STATUS graphics_splash(const uint8_t *content, size_t len);
