@@ -32,15 +32,6 @@ static int update_rules_map(
 
         assert(map_fd >= 0);
 
-        if (!head) {
-                static const struct socket_bind_rule val = {
-                        .address_family = SOCKET_BIND_RULE_AF_MATCH_NOTHING,
-                };
-
-                if (sym_bpf_map_update_elem(map_fd, &i, &val, BPF_ANY) != 0)
-                        return -errno;
-        }
-
         LIST_FOREACH(socket_bind_items, item, head) {
                 struct socket_bind_rule val = {
                         .address_family = (uint32_t) item->address_family,
@@ -165,7 +156,7 @@ static int socket_bind_install_impl(Unit *u) {
         _cleanup_(bpf_link_freep) struct bpf_link *ipv4 = NULL, *ipv6 = NULL;
         _cleanup_(socket_bind_bpf_freep) struct socket_bind_bpf *obj = NULL;
         _cleanup_free_ char *cgroup_path = NULL;
-        _cleanup_close_ int cgroup_fd = -1;
+        _cleanup_close_ int cgroup_fd = -EBADF;
         CGroupContext *cc;
         int r;
 
