@@ -29,10 +29,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 
         int alg = h->alg;
 
-        /* We don't want to fill the logs with messages about parse errors.
-         * Disable most logging if not running standalone */
-        if (!getenv("SYSTEMD_LOG_LEVEL"))
-                log_set_max_level(LOG_CRIT);
+        fuzz_setup_logging();
 
         log_info("Using compression %s, data size=%zu",
                  compression_to_string(alg),
@@ -45,7 +42,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         }
 
         size_t csize;
-        r = compress_blob_explicit(alg, h->data, data_len, buf, size, &csize);
+        r = compress_blob(alg, h->data, data_len, buf, size, &csize);
         if (r < 0) {
                 log_error_errno(r, "Compression failed: %m");
                 return 0;
@@ -63,7 +60,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         size_t sw_len = MIN(data_len - 1, h->sw_len);
 
         r = decompress_startswith(alg, buf, csize, &buf2, h->data, sw_len, h->data[sw_len]);
-        assert_se(r >= 0);
+        assert_se(r > 0);
 
         return 0;
 }
