@@ -24,22 +24,20 @@ static int property_get_auto_login(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
         assert(bus);
         assert(reply);
-        assert(m);
 
         r = sd_bus_message_open_container(reply, 'a', "(sso)");
         if (r < 0)
                 return r;
 
         HASHMAP_FOREACH(h, m->homes_by_name) {
-                _cleanup_(strv_freep) char **seats = NULL;
+                _cleanup_strv_free_ char **seats = NULL;
                 _cleanup_free_ char *home_path = NULL;
-                char **s;
 
                 r = home_auto_login(h, &seats);
                 if (r < 0) {
@@ -70,12 +68,11 @@ static int method_get_home_by_name(
 
         _cleanup_free_ char *path = NULL;
         const char *user_name;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = sd_bus_message_read(message, "s", &user_name);
         if (r < 0)
@@ -108,13 +105,12 @@ static int method_get_home_by_uid(
                 sd_bus_error *error) {
 
         _cleanup_free_ char *path = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         uint32_t uid;
         int r;
         Home *h;
 
         assert(message);
-        assert(m);
 
         r = sd_bus_message_read(message, "u", &uid);
         if (r < 0)
@@ -149,12 +145,11 @@ static int method_list_homes(
                 sd_bus_error *error) {
 
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *reply = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = sd_bus_message_new_method_return(message, &reply);
         if (r < 0)
@@ -198,14 +193,13 @@ static int method_get_user_record_by_name(
                 sd_bus_error *error) {
 
         _cleanup_free_ char *json = NULL, *path = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         const char *user_name;
         bool incomplete;
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = sd_bus_message_read(message, "s", &user_name);
         if (r < 0)
@@ -238,14 +232,13 @@ static int method_get_user_record_by_uid(
                 sd_bus_error *error) {
 
         _cleanup_free_ char *json = NULL, *path = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         bool incomplete;
         uint32_t uid;
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = sd_bus_message_read(message, "u", &uid);
         if (r < 0)
@@ -391,12 +384,11 @@ static int method_register_home(
                 sd_bus_error *error) {
 
         _cleanup_(user_record_unrefp) UserRecord *hr = NULL;
-        Manager *m = userdata;
-        Home *h;
+        Manager *m = ASSERT_PTR(userdata);
+        _cleanup_(home_freep) Home *h = NULL;
         int r;
 
         assert(message);
-        assert(m);
 
         r = bus_message_read_home_record(message, USER_RECORD_LOAD_EMBEDDED|USER_RECORD_PERMISSIVE, &hr, error);
         if (r < 0)
@@ -421,10 +413,10 @@ static int method_register_home(
                 return r;
 
         r = home_save_record(h);
-        if (r < 0) {
-                home_free(h);
+        if (r < 0)
                 return r;
-        }
+
+        TAKE_PTR(h);
 
         return sd_bus_reply_method_return(message, NULL);
 }
@@ -439,12 +431,11 @@ static int method_create_home(
                 sd_bus_error *error) {
 
         _cleanup_(user_record_unrefp) UserRecord *hr = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = bus_message_read_home_record(message, USER_RECORD_REQUIRE_REGULAR|USER_RECORD_ALLOW_SECRET|USER_RECORD_ALLOW_PRIVILEGED|USER_RECORD_ALLOW_PER_MACHINE|USER_RECORD_ALLOW_SIGNATURE, &hr, error);
         if (r < 0)
@@ -506,12 +497,11 @@ static int method_authenticate_home(sd_bus_message *message, void *userdata, sd_
 
 static int method_update_home(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(user_record_unrefp) UserRecord *hr = NULL;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
         assert(message);
-        assert(m);
 
         r = bus_message_read_home_record(message, USER_RECORD_REQUIRE_REGULAR|USER_RECORD_ALLOW_SECRET|USER_RECORD_ALLOW_PRIVILEGED|USER_RECORD_ALLOW_PER_MACHINE|USER_RECORD_ALLOW_SIGNATURE|USER_RECORD_PERMISSIVE, &hr, error);
         if (r < 0)
@@ -557,12 +547,10 @@ static int method_release_home(sd_bus_message *message, void *userdata, sd_bus_e
 static int method_lock_all_homes(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(operation_unrefp) Operation *o = NULL;
         bool waiting = false;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
-        assert(m);
-
         /* This is called from logind when we are preparing for system suspend. We enqueue a lock operation
          * for every suitable home we have and only when all of them completed we send a reply indicating
          * completion. */
@@ -600,12 +588,10 @@ static int method_lock_all_homes(sd_bus_message *message, void *userdata, sd_bus
 static int method_deactivate_all_homes(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(operation_unrefp) Operation *o = NULL;
         bool waiting = false;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Home *h;
         int r;
 
-        assert(m);
-
         /* This is called from systemd-homed-activate.service's ExecStop= command to ensure that all home
          * directories are shutdown before the system goes down. Note that we don't do this from
          * systemd-homed.service itself since we want to allow restarting of it without tearing down all home
@@ -635,179 +621,162 @@ static int method_deactivate_all_homes(sd_bus_message *message, void *userdata,
         return sd_bus_reply_method_return(message, NULL);
 }
 
+static int method_rebalance(sd_bus_message *message, void *userdata, sd_bus_error *error) {
+        Manager *m = ASSERT_PTR(userdata);
+        int r;
+
+        r = manager_schedule_rebalance(m, /* immediately= */ true);
+        if (r == 0)
+                return sd_bus_reply_method_errorf(message, BUS_ERROR_REBALANCE_NOT_NEEDED, "No home directories need rebalancing.");
+        if (r < 0)
+                return r;
+
+        /* Keep a reference to this message, so that we can reply to it once we are done */
+        r = set_ensure_put(&m->rebalance_queued_method_calls, &bus_message_hash_ops, message);
+        if (r < 0)
+                return log_error_errno(r, "Failed to track rebalance bus message: %m");
+
+        sd_bus_message_ref(message);
+        return 1;
+}
+
 static const sd_bus_vtable manager_vtable[] = {
         SD_BUS_VTABLE_START(0),
 
         SD_BUS_PROPERTY("AutoLogin", "a(sso)", property_get_auto_login, 0, SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
 
-        SD_BUS_METHOD_WITH_NAMES("GetHomeByName",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 "usussso",
-                                 SD_BUS_PARAM(uid)
-                                 SD_BUS_PARAM(home_state)
-                                 SD_BUS_PARAM(gid)
-                                 SD_BUS_PARAM(real_name)
-                                 SD_BUS_PARAM(home_directory)
-                                 SD_BUS_PARAM(shell)
-                                 SD_BUS_PARAM(bus_path),
-                                 method_get_home_by_name,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("GetHomeByUID",
-                                 "u",
-                                 SD_BUS_PARAM(uid),
-                                 "ssussso",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(home_state)
-                                 SD_BUS_PARAM(gid)
-                                 SD_BUS_PARAM(real_name)
-                                 SD_BUS_PARAM(home_directory)
-                                 SD_BUS_PARAM(shell)
-                                 SD_BUS_PARAM(bus_path),
-                                 method_get_home_by_uid,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("GetUserRecordByName",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 "sbo",
-                                 SD_BUS_PARAM(user_record)
-                                 SD_BUS_PARAM(incomplete)
-                                 SD_BUS_PARAM(bus_path),
-                                 method_get_user_record_by_name,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
-        SD_BUS_METHOD_WITH_NAMES("GetUserRecordByUID",
-                                 "u",
-                                 SD_BUS_PARAM(uid),
-                                 "sbo",
-                                 SD_BUS_PARAM(user_record)
-                                 SD_BUS_PARAM(incomplete)
-                                 SD_BUS_PARAM(bus_path),
-                                 method_get_user_record_by_uid,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
-        SD_BUS_METHOD_WITH_NAMES("ListHomes",
-                                 NULL,,
-                                 "a(susussso)",
-                                 SD_BUS_PARAM(home_areas),
-                                 method_list_homes,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("GetHomeByName",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_RESULT("u", uid,
+                                              "s", home_state,
+                                              "u", gid,
+                                              "s", real_name,
+                                              "s", home_directory,
+                                              "s", shell,
+                                              "o", bus_path),
+                                method_get_home_by_name,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("GetHomeByUID",
+                                SD_BUS_ARGS("u", uid),
+                                SD_BUS_RESULT("s", user_name,
+                                              "s", home_state,
+                                              "u", gid,
+                                              "s", real_name,
+                                              "s", home_directory,
+                                              "s", shell,
+                                              "o", bus_path),
+                                method_get_home_by_uid,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("GetUserRecordByName",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_RESULT("s", user_record, "b", incomplete, "o", bus_path),
+                                method_get_user_record_by_name,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("GetUserRecordByUID",
+                                SD_BUS_ARGS("u", uid),
+                                SD_BUS_RESULT("s", user_record, "b", incomplete, "o", bus_path),
+                                method_get_user_record_by_uid,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("ListHomes",
+                                SD_BUS_NO_ARGS,
+                                SD_BUS_RESULT("a(susussso)", home_areas),
+                                method_list_homes,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
         /* The following methods directly execute an operation on a home area, without ref-counting, queueing
          * or anything, and are accessible through homectl. */
-        SD_BUS_METHOD_WITH_NAMES("ActivateHome",
-                                 "ss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_activate_home,
-                                 SD_BUS_VTABLE_SENSITIVE),
-        SD_BUS_METHOD_WITH_NAMES("DeactivateHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 NULL,,
-                                 method_deactivate_home,
-                                 0),
+        SD_BUS_METHOD_WITH_ARGS("ActivateHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_activate_home,
+                                SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("DeactivateHome",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_NO_RESULT,
+                                method_deactivate_home,
+                                0),
 
         /* Add the JSON record to homed, but don't create actual $HOME */
-        SD_BUS_METHOD_WITH_NAMES("RegisterHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_record),
-                                 NULL,,
-                                 method_register_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("RegisterHome",
+                                SD_BUS_ARGS("s", user_record),
+                                SD_BUS_NO_RESULT,
+                                method_register_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
         /* Remove the JSON record from homed, but don't remove actual $HOME  */
-        SD_BUS_METHOD_WITH_NAMES("UnregisterHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 NULL,,
-                                 method_unregister_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("UnregisterHome",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_NO_RESULT,
+                                method_unregister_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
         /* Add JSON record, and create $HOME for it */
-        SD_BUS_METHOD_WITH_NAMES("CreateHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_record),
-                                 NULL,,
-                                 method_create_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("CreateHome",
+                                SD_BUS_ARGS("s", user_record),
+                                SD_BUS_NO_RESULT,
+                                method_create_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
         /* Create $HOME for already registered JSON entry */
-        SD_BUS_METHOD_WITH_NAMES("RealizeHome",
-                                 "ss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_realize_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("RealizeHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_realize_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
         /* Remove the JSON record and remove $HOME */
-        SD_BUS_METHOD_WITH_NAMES("RemoveHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 NULL,,
-                                 method_remove_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("RemoveHome",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_NO_RESULT,
+                                method_remove_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
         /* Investigate $HOME and propagate contained JSON record into our database */
-        SD_BUS_METHOD_WITH_NAMES("FixateHome",
-                                 "ss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_fixate_home,
-                                 SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("FixateHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_fixate_home,
+                                SD_BUS_VTABLE_SENSITIVE),
 
         /* Just check credentials */
-        SD_BUS_METHOD_WITH_NAMES("AuthenticateHome",
-                                 "ss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_authenticate_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("AuthenticateHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_authenticate_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
         /* Update the JSON record of existing user */
-        SD_BUS_METHOD_WITH_NAMES("UpdateHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_record),
-                                 NULL,,
-                                 method_update_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("UpdateHome",
+                                SD_BUS_ARGS("s", user_record),
+                                SD_BUS_NO_RESULT,
+                                method_update_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
-        SD_BUS_METHOD_WITH_NAMES("ResizeHome",
-                                 "sts",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(size)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_resize_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("ResizeHome",
+                                SD_BUS_ARGS("s", user_name, "t", size, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_resize_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
-        SD_BUS_METHOD_WITH_NAMES("ChangePasswordHome",
-                                 "sss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(new_secret)
-                                 SD_BUS_PARAM(old_secret),
-                                 NULL,,
-                                 method_change_password_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("ChangePasswordHome",
+                                SD_BUS_ARGS("s", user_name, "s", new_secret, "s", old_secret),
+                                SD_BUS_NO_RESULT,
+                                method_change_password_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
 
         /* Prepare active home for system suspend: flush out passwords, suspend access */
-        SD_BUS_METHOD_WITH_NAMES("LockHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 NULL,,
-                                 method_lock_home,
-                                 0),
+        SD_BUS_METHOD_WITH_ARGS("LockHome",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_NO_RESULT,
+                                method_lock_home,
+                                0),
 
         /* Make $HOME usable after system resume again */
-        SD_BUS_METHOD_WITH_NAMES("UnlockHome",
-                                 "ss",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret),
-                                 NULL,,
-                                 method_unlock_home,
-                                 SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("UnlockHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret),
+                                SD_BUS_NO_RESULT,
+                                method_unlock_home,
+                                SD_BUS_VTABLE_SENSITIVE),
 
         /* The following methods implement ref-counted activation, and are what the PAM module and "homectl
          * with" use. In contrast to the methods above which fail if an operation is already being executed
@@ -816,33 +785,26 @@ static const sd_bus_vtable manager_vtable[] = {
          * the state of the home area, so that the end result is always the same (i.e. the home directory is
          * accessible), and we always validate the specified passwords. RefHome() will not authenticate, and
          * thus only works if the home area is already active. */
-        SD_BUS_METHOD_WITH_NAMES("AcquireHome",
-                                 "ssb",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(secret)
-                                 SD_BUS_PARAM(please_suspend),
-                                 "h",
-                                 SD_BUS_PARAM(send_fd),
-                                 method_acquire_home,
-                                 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
-        SD_BUS_METHOD_WITH_NAMES("RefHome",
-                                 "sb",
-                                 SD_BUS_PARAM(user_name)
-                                 SD_BUS_PARAM(please_suspend),
-                                 "h",
-                                 SD_BUS_PARAM(send_fd),
-                                 method_ref_home,
-                                 0),
-        SD_BUS_METHOD_WITH_NAMES("ReleaseHome",
-                                 "s",
-                                 SD_BUS_PARAM(user_name),
-                                 NULL,,
-                                 method_release_home,
-                                 0),
+        SD_BUS_METHOD_WITH_ARGS("AcquireHome",
+                                SD_BUS_ARGS("s", user_name, "s", secret, "b", please_suspend),
+                                SD_BUS_RESULT("h", send_fd),
+                                method_acquire_home,
+                                SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_SENSITIVE),
+        SD_BUS_METHOD_WITH_ARGS("RefHome",
+                                SD_BUS_ARGS("s", user_name, "b", please_suspend),
+                                SD_BUS_RESULT("h", send_fd),
+                                method_ref_home,
+                                0),
+        SD_BUS_METHOD_WITH_ARGS("ReleaseHome",
+                                SD_BUS_ARGS("s", user_name),
+                                SD_BUS_NO_RESULT,
+                                method_release_home,
+                                0),
 
         /* An operation that acts on all homes that allow it */
         SD_BUS_METHOD("LockAllHomes", NULL, NULL, method_lock_all_homes, 0),
         SD_BUS_METHOD("DeactivateAllHomes", NULL, NULL, method_deactivate_all_homes, 0),
+        SD_BUS_METHOD("Rebalance", NULL, NULL, method_rebalance, 0),
 
         SD_BUS_VTABLE_END
 };
@@ -855,11 +817,9 @@ const BusObjectImplementation manager_object = {
 };
 
 static int on_deferred_auto_login(sd_event_source *s, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         int r;
 
-        assert(m);
-
         m->deferred_auto_login_event_source = sd_event_source_disable_unref(m->deferred_auto_login_event_source);
 
         r = sd_bus_emit_properties_changed(
