@@ -142,9 +142,7 @@ int home_move_mount(const char *mount_suffix, const char *target) {
         } else
                 d = HOME_RUNTIME_WORK_DIR;
 
-        r = mkdir_p(target, 0700);
-        if (r < 0)
-                return log_error_errno(r, "Failed to create directory '%s': %m", target);
+        (void) mkdir_p(target, 0700);
 
         r = mount_nofollow_verbose(LOG_ERR, d, target, NULL, MS_BIND, NULL);
         if (r < 0)
@@ -186,9 +184,9 @@ static int append_identity_range(char **text, uid_t start, uid_t next_start, uid
                           exclude + 1, exclude + 1, next_start - exclude - 1);
 }
 
-static int make_userns(uid_t stored_uid, uid_t exposed_uid) {
+static int make_home_userns(uid_t stored_uid, uid_t exposed_uid) {
         _cleanup_free_ char *text = NULL;
-        _cleanup_close_ int userns_fd = -1;
+        _cleanup_close_ int userns_fd = -EBADF;
         int r;
 
         assert(uid_is_valid(stored_uid));
@@ -228,7 +226,7 @@ static int make_userns(uid_t stored_uid, uid_t exposed_uid) {
                 return log_oom();
 
         /* Leave everything else unmapped, starting from UID_NOBODY itself. Specifically, this means the
-         * whole space outside of 16bit remains unmapped */
+         * whole space outside of 16-bit remains unmapped */
 
         log_debug("Creating userns with mapping:\n%s", text);
 
@@ -240,7 +238,7 @@ static int make_userns(uid_t stored_uid, uid_t exposed_uid) {
 }
 
 int home_shift_uid(int dir_fd, const char *target, uid_t stored_uid, uid_t exposed_uid, int *ret_mount_fd) {
-        _cleanup_close_ int mount_fd = -1, userns_fd = -1;
+        _cleanup_close_ int mount_fd = -EBADF, userns_fd = -EBADF;
         int r;
 
         assert(dir_fd >= 0);
@@ -263,7 +261,7 @@ int home_shift_uid(int dir_fd, const char *target, uid_t stored_uid, uid_t expos
                         log_debug_errno(errno, "The open_tree() syscall is not supported, not setting up UID shift mount: %m");
 
                         if (ret_mount_fd)
-                                *ret_mount_fd = -1;
+                                *ret_mount_fd = -EBADF;
 
                         return 0;
                 }
@@ -271,7 +269,7 @@ int home_shift_uid(int dir_fd, const char *target, uid_t stored_uid, uid_t expos
                 return log_error_errno(errno, "Failed to open tree of home directory: %m");
         }
 
-        userns_fd = make_userns(stored_uid, exposed_uid);
+        userns_fd = make_home_userns(stored_uid, exposed_uid);
         if (userns_fd < 0)
                 return userns_fd;
 
@@ -286,7 +284,7 @@ int home_shift_uid(int dir_fd, const char *target, uid_t stored_uid, uid_t expos
                         log_debug_errno(errno, "UID/GID mapping for shifted mount not available, not setting it up: %m");
 
                         if (ret_mount_fd)
-                                *ret_mount_fd = -1;
+                                *ret_mount_fd = -EBADF;
 
                         return 0;
                 }
