@@ -232,7 +232,7 @@ static void raw_pull_report_progress(RawPull *i, RawProgress p) {
                 assert_not_reached();
         }
 
-        sd_notifyf(false, "X_IMPORT_PROGRESS=%u", percent);
+        sd_notifyf(false, "X_IMPORT_PROGRESS=%u%%", percent);
         log_debug("Combined progress %u%%", percent);
 }
 
@@ -327,7 +327,6 @@ static int raw_pull_copy_auxiliary_file(
                         *path,
                         local,
                         0644,
-                        0, 0,
                         COPY_REFLINK |
                         (FLAGS_SET(i->flags, PULL_FORCE) ? COPY_REPLACE : 0) |
                         (FLAGS_SET(i->flags, PULL_SYNC) ? COPY_FSYNC_FULL : 0));
@@ -371,7 +370,7 @@ static int raw_pull_make_local_copy(RawPull *i) {
                 assert(i->raw_job->disk_fd >= 0);
                 assert(i->offset == UINT64_MAX);
 
-                if (lseek(i->raw_job->disk_fd, SEEK_SET, 0) == (off_t) -1)
+                if (lseek(i->raw_job->disk_fd, SEEK_SET, 0) < 0)
                         return log_error_errno(errno, "Failed to seek to beginning of vendor image: %m");
         }
 
@@ -396,7 +395,7 @@ static int raw_pull_make_local_copy(RawPull *i) {
                 return log_error_errno(r, "Failed to make writable copy of image: %m");
 
         (void) copy_times(i->raw_job->disk_fd, dfd, COPY_CRTIME);
-        (void) copy_xattr(i->raw_job->disk_fd, dfd, 0);
+        (void) copy_xattr(i->raw_job->disk_fd, NULL, dfd, NULL, 0);
 
         dfd = safe_close(dfd);
 
