@@ -10,10 +10,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         if (size == 0)
                 return 0;
 
-        /* We don't want to fill the logs with assert warnings.
-         * Disable most logging if not running standalone */
-        if (!getenv("SYSTEMD_LOG_LEVEL"))
-                log_set_max_level(LOG_CRIT);
+        fuzz_setup_logging();
 
         dummy_server_init(&s, data, size);
         dev_kmsg_record(&s, s.buffer, size);
