@@ -7,6 +7,7 @@
 #include "sd-dhcp-client.c"
 
 #include "alloc-util.h"
+#include "dhcp-network.h"
 #include "fuzz.h"
 
 int dhcp_network_bind_raw_socket(
@@ -53,8 +54,9 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         _cleanup_(sd_event_unrefp) sd_event *e = NULL;
         int res, r;
 
-        if (!getenv("SYSTEMD_LOG_LEVEL"))
-                log_set_max_level(LOG_CRIT);
+        assert_se(setenv("SYSTEMD_NETWORK_TEST_MODE", "1", 1) >= 0);
+
+        fuzz_setup_logging();
 
         r = sd_dhcp_client_new(&client, false);
         assert_se(r >= 0);
@@ -67,13 +69,13 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 
         assert_se(sd_dhcp_client_set_ifindex(client, 42) >= 0);
         assert_se(sd_dhcp_client_set_mac(client, mac_addr, bcast_addr, ETH_ALEN, ARPHRD_ETHER) >= 0);
-        dhcp_client_set_test_mode(client, true);
 
         res = sd_dhcp_client_start(client);
         assert_se(IN_SET(res, 0, -EINPROGRESS));
         client->xid = 2;
+        client->state = DHCP_STATE_SELECTING;
 
-        (void) client_handle_offer(client, (DHCPMessage*) data, size);
+        (void) client_handle_offer_or_rapid_ack(client, (DHCPMessage*) data, size, NULL);
 
         assert_se(sd_dhcp_client_stop(client) >= 0);
 
