@@ -10,7 +10,7 @@
 #include "fd-util.h"
 #include "fuzz.h"
 
-static int test_dhcp_fd[2] = PIPE_EBADF;
+static int test_dhcp_fd[2] = EBADF_PAIR;
 
 int dhcp6_network_send_udp_socket(int s, struct in6_addr *server_address, const void *packet, size_t len) {
         return len;
@@ -73,8 +73,9 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         struct in6_addr hint = { { { 0x3f, 0xfe, 0x05, 0x01, 0xff, 0xff, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } } };
         static const char *v1_data = "hogehoge", *v2_data = "foobar";
 
-        if (!getenv("SYSTEMD_LOG_LEVEL"))
-                log_set_max_level(LOG_CRIT);
+        assert_se(setenv("SYSTEMD_NETWORK_TEST_MODE", "1", 1) >= 0);
+
+        fuzz_setup_logging();
 
         if (outside_size_range(size, 0, 65536))
                 return 0;
@@ -84,7 +85,6 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         assert_se(sd_dhcp6_client_attach_event(client, e, 0) >= 0);
         assert_se(sd_dhcp6_client_set_ifindex(client, 42) >= 0);
         assert_se(sd_dhcp6_client_set_local_address(client, &address) >= 0);
-        dhcp6_client_set_test_mode(client, true);
 
         /* Used when sending message. */
         assert_se(sd_dhcp6_client_set_fqdn(client, "example.com") == 1);
