@@ -23,10 +23,11 @@ struct sd_ndisc_router {
 
         uint64_t flags;
         unsigned preference;
-        uint16_t lifetime;
+        uint64_t lifetime_usec;
 
         uint8_t hop_limit;
         uint32_t mtu;
+        uint64_t icmp6_ratelimit_usec;
 };
 
 static inline void* NDISC_ROUTER_RAW(const sd_ndisc_router *rt) {
@@ -45,4 +46,4 @@ static inline size_t NDISC_ROUTER_OPTION_LENGTH(const sd_ndisc_router *rt) {
 }
 
 sd_ndisc_router *ndisc_router_new(size_t raw_size);
-int ndisc_router_parse(sd_ndisc_router *rt);
+int ndisc_router_parse(sd_ndisc *nd, sd_ndisc_router *rt);
