@@ -18,8 +18,9 @@
 
 #include "alloc-util.h"
 #include "dhcp-identifier.h"
-#include "dhcp-internal.h"
-#include "dhcp-protocol.h"
+#include "dhcp-network.h"
+#include "dhcp-option.h"
+#include "dhcp-packet.h"
 #include "ether-addr-util.h"
 #include "fd-util.h"
 #include "random-util.h"
@@ -45,7 +46,7 @@ static void test_request_basic(sd_event *e) {
         sd_dhcp_client *client;
 
         if (verbose)
-                log_info("* %s", __func__);
+                printf("* %s\n", __func__);
 
         /* Initialize client without Anonymize settings. */
         r = sd_dhcp_client_new(&client, false);
@@ -105,7 +106,7 @@ static void test_request_anonymize(sd_event *e) {
         sd_dhcp_client *client;
 
         if (verbose)
-                log_info("* %s", __func__);
+                printf("* %s\n", __func__);
 
         /* Initialize client with Anonymize settings. */
         r = sd_dhcp_client_new(&client, true);
@@ -137,7 +138,7 @@ static void test_checksum(void) {
         };
 
         if (verbose)
-                log_info("* %s", __func__);
+                printf("* %s\n", __func__);
 
         assert_se(dhcp_packet_checksum((uint8_t*)&buf, 20) == be16toh(0x78ae));
 }
@@ -168,7 +169,7 @@ static int check_options(uint8_t code, uint8_t len, const void *option, void *us
                 struct duid duid;
                 size_t duid_len;
 
-                assert_se(dhcp_identifier_set_duid_en(/* test_mode = */ true, &duid, &duid_len) >= 0);
+                assert_se(dhcp_identifier_set_duid_en(&duid, &duid_len) >= 0);
                 assert_se(dhcp_identifier_set_iaid(NULL, &hw_addr, /* legacy = */ true, &iaid) >= 0);
 
                 assert_se(len == sizeof(uint8_t) + sizeof(uint32_t) + duid_len);
@@ -270,7 +271,7 @@ static int test_discover_message_verify(size_t size, struct DHCPMessage *dhcp) {
         assert_se(res == DHCP_DISCOVER);
 
         if (verbose)
-                log_info("  recv DHCP Discover 0x%08x", be32toh(dhcp->xid));
+                printf("  recv DHCP Discover 0x%08x\n", be32toh(dhcp->xid));
 
         return 0;
 }
@@ -280,7 +281,7 @@ static void test_discover_message(sd_event *e) {
         int res, r;
 
         if (verbose)
-                log_info("* %s", __func__);
+                printf("* %s\n", __func__);
 
         r = sd_dhcp_client_new(&client, false);
         assert_se(r >= 0);
@@ -291,7 +292,6 @@ static void test_discover_message(sd_event *e) {
 
         assert_se(sd_dhcp_client_set_ifindex(client, 42) >= 0);
         assert_se(sd_dhcp_client_set_mac(client, hw_addr.bytes, bcast_addr.bytes, hw_addr.length, ARPHRD_ETHER) >= 0);
-        dhcp_client_set_test_mode(client, true);
 
         assert_se(sd_dhcp_client_set_request_option(client, 248) >= 0);
 
@@ -425,7 +425,7 @@ static int test_addr_acq_acquired(sd_dhcp_client *client, int event,
                          sizeof(addrs[0].s_addr)) == 0);
 
         if (verbose)
-                log_info("  DHCP address acquired");
+                printf("  DHCP address acquired\n");
 
         sd_event_exit(e, 0);
 
@@ -444,7 +444,7 @@ static int test_addr_acq_recv_request(size_t size, DHCPMessage *request) {
         assert_se(msg_bytes[size - 1] == SD_DHCP_OPTION_END);
 
         if (verbose)
-                log_info("  recv DHCP Request  0x%08x", be32toh(xid));
+                printf("  recv DHCP Request  0x%08x\n", be32toh(xid));
 
         memcpy(&test_addr_acq_ack[26], &udp_check, sizeof(udp_check));
         memcpy(&test_addr_acq_ack[32], &xid, sizeof(xid));
@@ -457,7 +457,7 @@ static int test_addr_acq_recv_request(size_t size, DHCPMessage *request) {
         assert_se(res == sizeof(test_addr_acq_ack));
 
         if (verbose)
-                log_info("  send DHCP Ack");
+                printf("  send DHCP Ack\n");
 
         return 0;
 };
@@ -475,7 +475,7 @@ static int test_addr_acq_recv_discover(size_t size, DHCPMessage *discover) {
         xid = discover->xid;
 
         if (verbose)
-                log_info("  recv DHCP Discover 0x%08x", be32toh(xid));
+                printf("  recv DHCP Discover 0x%08x\n", be32toh(xid));
 
         memcpy(&test_addr_acq_offer[26], &udp_check, sizeof(udp_check));
         memcpy(&test_addr_acq_offer[32], &xid, sizeof(xid));
@@ -488,7 +488,7 @@ static int test_addr_acq_recv_discover(size_t size, DHCPMessage *discover) {
         assert_se(res == sizeof(test_addr_acq_offer));
 
         if (verbose)
-                log_info("  sent DHCP Offer");
+                printf("  sent DHCP Offer\n");
 
         return 0;
 }
@@ -498,7 +498,7 @@ static void test_addr_acq(sd_event *e) {
         int res, r;
 
         if (verbose)
-                log_info("* %s", __func__);
+                printf("* %s\n", __func__);
 
         r = sd_dhcp_client_new(&client, false);
         assert_se(r >= 0);
@@ -509,14 +509,13 @@ static void test_addr_acq(sd_event *e) {
 
         assert_se(sd_dhcp_client_set_ifindex(client, 42) >= 0);
         assert_se(sd_dhcp_client_set_mac(client, hw_addr.bytes, bcast_addr.bytes, hw_addr.length, ARPHRD_ETHER) >= 0);
-        dhcp_client_set_test_mode(client, true);
 
         assert_se(sd_dhcp_client_set_callback(client, test_addr_acq_acquired, e) >= 0);
 
         callback_recv = test_addr_acq_recv_discover;
 
         assert_se(sd_event_add_time_relative(e, NULL, CLOCK_BOOTTIME,
-                                             30 * USEC_PER_SEC, 0,
+                                             2 * USEC_PER_SEC, 0,
                                              NULL, INT_TO_PTR(-ETIMEDOUT)) >= 0);
 
         res = sd_dhcp_client_start(client);
@@ -537,6 +536,8 @@ static void test_addr_acq(sd_event *e) {
 int main(int argc, char *argv[]) {
         _cleanup_(sd_event_unrefp) sd_event *e;
 
+        assert_se(setenv("SYSTEMD_NETWORK_TEST_MODE", "1", 1) >= 0);
+
         test_setup_logging(LOG_DEBUG);
 
         assert_se(sd_event_new(&e) >= 0);
