@@ -19,6 +19,7 @@
 #include "fd-util.h"
 #include "fs-util.h"
 #include "io-util.h"
+#include "iovec-util.h"
 #include "parse-util.h"
 #include "path-util.h"
 #include "process-util.h"
@@ -475,12 +476,6 @@ static int pid_notify_with_fds_internal(
         if (n_fds > 0 && !fds)
                 return -EINVAL;
 
-        /* Let's make sure the multiplications below don't overflow, and also return a recognizable error in
-         * case the caller tries to send more fds than the kernel limit. The kernel would return EINVAL which
-         * is not too useful I'd say. */
-        if (n_fds > SCM_MAX_FD)
-                return -E2BIG;
-
         e = getenv("NOTIFY_SOCKET");
         if (!e)
                 return 0;
@@ -590,27 +585,14 @@ static int pid_notify_with_fds_internal(
                         send_ucred = false;
                 } else {
                         /* Unless we're using SOCK_STREAM, we expect to write all the contents immediately. */
-                        if (type != SOCK_STREAM && (size_t) n < IOVEC_TOTAL_SIZE(msghdr.msg_iov, msghdr.msg_iovlen))
+                        if (type != SOCK_STREAM && (size_t) n < iovec_total_size(msghdr.msg_iov, msghdr.msg_iovlen))
                                 return -EIO;
 
                         /* Make sure we only send fds and ucred once, even if we're using SOCK_STREAM. */
                         msghdr.msg_control = NULL;
                         msghdr.msg_controllen = 0;
                 }
-        } while (!IOVEC_INCREMENT(msghdr.msg_iov, msghdr.msg_iovlen, n));
-
-        if (address.sockaddr.sa.sa_family == AF_VSOCK && IN_SET(type, SOCK_STREAM, SOCK_SEQPACKET)) {
-                /* For AF_VSOCK, we need to close the socket to signal the end of the message. */
-                if (shutdown(fd, SHUT_WR) < 0)
-                        return log_debug_errno(errno, "Failed to shutdown notify socket: %m");
-
-                char c;
-                n = recv(fd, &c, sizeof(c), MSG_NOSIGNAL);
-                if (n < 0)
-                        return log_debug_errno(errno, "Failed to wait for EOF on notify socket: %m");
-                if (n > 0)
-                        return log_debug_errno(SYNTHETIC_ERRNO(EPROTO), "Unexpectedly received data on notify socket.");
-        }
+        } while (!iovec_increment(msghdr.msg_iov, msghdr.msg_iovlen, n));
 
         return 1;
 }
@@ -633,7 +615,7 @@ _public_ int sd_pid_notify_with_fds(
 }
 
 _public_ int sd_pid_notify_barrier(pid_t pid, int unset_environment, uint64_t timeout) {
-        _cleanup_close_pair_ int pipe_fd[2] = PIPE_EBADF;
+        _cleanup_close_pair_ int pipe_fd[2] = EBADF_PAIR;
         int r;
 
         if (pipe2(pipe_fd, O_CLOEXEC) < 0)
@@ -733,18 +715,17 @@ _public_ int sd_pid_notifyf_with_fds(
 }
 
 _public_ int sd_booted(void) {
-        int r;
+        /* We test whether the runtime unit file directory has been
+         * created. This takes place in mount-setup.c, so is
+         * guaranteed to happen very early during boot. */
 
-        /* We test whether the runtime unit file directory has been created. This takes place in mount-setup.c,
-         * so is guaranteed to happen very early during boot. */
-
-        r = laccess("/run/systemd/system/", F_OK);
-        if (r >= 0)
+        if (laccess("/run/systemd/system/", F_OK) >= 0)
                 return true;
-        if (r == -ENOENT)
+
+        if (errno == ENOENT)
                 return false;
 
-        return r;
+        return -errno;
 }
 
 _public_ int sd_watchdog_enabled(int unset_environment, uint64_t *usec) {
