@@ -194,7 +194,6 @@ static void test_sd_device_one(sd_device *d) {
 
 TEST(sd_device_enumerator_devices) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
-        sd_device *d;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
         assert_se(sd_device_enumerator_allow_uninitialized(e) >= 0);
@@ -211,7 +210,6 @@ TEST(sd_device_enumerator_devices) {
 
 TEST(sd_device_enumerator_subsystems) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
-        sd_device *d;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
         assert_se(sd_device_enumerator_allow_uninitialized(e) >= 0);
@@ -227,7 +225,7 @@ static void test_sd_device_enumerator_filter_subsystem_one(
 
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
         unsigned n_new_dev = 0, n_removed_dev = 0;
-        sd_device *d;
+        sd_device *dev;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
         assert_se(sd_device_enumerator_add_match_subsystem(e, subsystem, true) >= 0);
@@ -248,14 +246,14 @@ static void test_sd_device_enumerator_filter_subsystem_one(
                 assert_se(!sd_device_unref(t));
         }
 
-        HASHMAP_FOREACH(d, h) {
+        HASHMAP_FOREACH(dev, h) {
                 const char *syspath;
 
-                assert_se(sd_device_get_syspath(d, &syspath) >= 0);
+                assert_se(sd_device_get_syspath(dev, &syspath) >= 0);
                 log_warning("Device removed: subsystem:%s syspath:%s", subsystem, syspath);
                 n_removed_dev++;
 
-                assert_se(!sd_device_unref(d));
+                assert_se(!sd_device_unref(dev));
         }
 
         hashmap_free(h);
@@ -266,9 +264,8 @@ static void test_sd_device_enumerator_filter_subsystem_one(
 
 static bool test_sd_device_enumerator_filter_subsystem_trial(void) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
-        _cleanup_(hashmap_freep) Hashmap *subsystems = NULL;
+        _cleanup_hashmap_free_ Hashmap *subsystems = NULL;
         unsigned n_new_dev = 0, n_removed_dev = 0;
-        sd_device *d;
         Hashmap *h;
         char *s;
 
@@ -402,10 +399,35 @@ TEST(sd_device_enumerator_add_match_property) {
         assert_se(ifindex == 1);
 }
 
+TEST(sd_device_enumerator_add_match_property_required) {
+        _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
+        sd_device *dev;
+        int ifindex;
+
+        assert_se(sd_device_enumerator_new(&e) >= 0);
+        assert_se(sd_device_enumerator_allow_uninitialized(e) >= 0);
+        assert_se(sd_device_enumerator_add_match_subsystem(e, "net", true) >= 0);
+        assert_se(sd_device_enumerator_add_match_sysattr(e, "ifindex", "1", true) >= 0);
+        assert_se(sd_device_enumerator_add_match_property_required(e, "IFINDE*", "1*") >= 0);
+
+        /* Only one required match which should be satisfied. */
+        dev = sd_device_enumerator_get_device_first(e);
+        assert_se(dev);
+        assert_se(sd_device_get_ifindex(dev, &ifindex) >= 0);
+        assert_se(ifindex == 1);
+
+        /* Now let's add a bunch of garbage properties which should not be satisfied. */
+        assert_se(sd_device_enumerator_add_match_property_required(e, "IFINDE*", "hoge") >= 0);
+        assert_se(sd_device_enumerator_add_match_property_required(e, "IFINDE*", NULL) >= 0);
+        assert_se(sd_device_enumerator_add_match_property_required(e, "AAAAA", "BBBB") >= 0);
+        assert_se(sd_device_enumerator_add_match_property_required(e, "FOOOO", NULL) >= 0);
+
+        assert_se(!sd_device_enumerator_get_device_first(e));
+}
+
 static void check_parent_match(sd_device_enumerator *e, sd_device *dev) {
         const char *syspath;
         bool found = false;
-        sd_device *d;
 
         assert_se(sd_device_get_syspath(dev, &syspath) >= 0);
 
@@ -429,7 +451,6 @@ static void check_parent_match(sd_device_enumerator *e, sd_device *dev) {
 
 TEST(sd_device_enumerator_add_match_parent) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
-        sd_device *dev;
         int r;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
@@ -475,7 +496,6 @@ TEST(sd_device_enumerator_add_match_parent) {
 
 TEST(sd_device_get_child) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
-        sd_device *dev;
         int r;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
@@ -490,7 +510,7 @@ TEST(sd_device_get_child) {
 
         FOREACH_DEVICE(e, dev) {
                 const char *syspath, *parent_syspath, *expected_suffix, *suffix;
-                sd_device *parent, *child;
+                sd_device *parent;
                 bool found = false;
 
                 assert_se(sd_device_get_syspath(dev, &syspath) >= 0);
@@ -580,7 +600,6 @@ TEST(sd_device_new_from_nulstr) {
 TEST(sd_device_new_from_path) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
         _cleanup_(rm_rf_physical_and_freep) char *tmpdir = NULL;
-        sd_device *dev;
         int r;
 
         assert_se(mkdtemp_malloc("/tmp/test-sd-device.XXXXXXX", &tmpdir) >= 0);
