@@ -4,9 +4,4 @@
 #include <stdbool.h>
 
 int journal_fd_nonblock(bool nonblock);
-
-#if VALGRIND
 void close_journal_fd(void);
-#else
-static inline void close_journal_fd(void) {}
-#endif
