@@ -7,12 +7,12 @@
 
 #include "sd-netlink.h"
 
-#include "io-util.h"
+#include "iovec-util.h"
 #include "netlink-internal.h"
 #include "netlink-types.h"
 #include "netlink-util.h"
 
-static bool nfproto_is_valid(int nfproto) {
+bool nfproto_is_valid(int nfproto) {
         return IN_SET(nfproto,
                       NFPROTO_UNSPEC,
                       NFPROTO_INET,
@@ -20,8 +20,7 @@ static bool nfproto_is_valid(int nfproto) {
                       NFPROTO_ARP,
                       NFPROTO_NETDEV,
                       NFPROTO_BRIDGE,
-                      NFPROTO_IPV6,
-                      NFPROTO_DECNET);
+                      NFPROTO_IPV6);
 }
 
 int sd_nfnl_message_new(sd_netlink *nfnl, sd_netlink_message **ret, int nfproto, uint16_t subsys, uint16_t msg_type, uint16_t flags) {
@@ -184,7 +183,7 @@ int sd_nfnl_call_batch(
 
         _cleanup_free_ sd_netlink_message **replies = NULL;
         _cleanup_free_ uint32_t *serials = NULL;
-        int k, r;
+        int r;
 
         assert_return(nfnl, -EINVAL);
         assert_return(!netlink_pid_changed(nfnl), -ECHILD);
@@ -201,11 +200,9 @@ int sd_nfnl_call_batch(
         if (r < 0)
                 return r;
 
-        for (size_t i = 0; i < n_messages; i++) {
-                k = sd_netlink_read(nfnl, serials[i], usec, ret_messages ? replies + i : NULL);
-                if (k < 0 && r >= 0)
-                        r = k;
-        }
+        for (size_t i = 0; i < n_messages; i++)
+                RET_GATHER(r,
+                           sd_netlink_read(nfnl, serials[i], usec, ret_messages ? replies + i : NULL));
         if (r < 0)
                 return r;
 
