@@ -16,7 +16,7 @@ Neighbor *neighbor_free(Neighbor *neighbor) {
 
         if (neighbor->network) {
                 assert(neighbor->section);
-                hashmap_remove(neighbor->network->neighbors_by_section, neighbor->section);
+                ordered_hashmap_remove(neighbor->network->neighbors_by_section, neighbor->section);
         }
 
         config_section_free(neighbor->section);
@@ -43,7 +43,7 @@ static int neighbor_new_static(Network *network, const char *filename, unsigned
         if (r < 0)
                 return r;
 
-        neighbor = hashmap_get(network->neighbors_by_section, n);
+        neighbor = ordered_hashmap_get(network->neighbors_by_section, n);
         if (neighbor) {
                 *ret = TAKE_PTR(neighbor);
                 return 0;
@@ -60,7 +60,7 @@ static int neighbor_new_static(Network *network, const char *filename, unsigned
                 .source = NETWORK_CONFIG_SOURCE_STATIC,
         };
 
-        r = hashmap_ensure_put(&network->neighbors_by_section, &config_section_hash_ops, neighbor->section, neighbor);
+        r = ordered_hashmap_ensure_put(&network->neighbors_by_section, &config_section_hash_ops, neighbor->section, neighbor);
         if (r < 0)
                 return r;
 
@@ -92,18 +92,13 @@ static void neighbor_hash_func(const Neighbor *neighbor, struct siphash *state)
 
         siphash24_compress(&neighbor->family, sizeof(neighbor->family), state);
 
-        switch (neighbor->family) {
-        case AF_INET:
-        case AF_INET6:
-                /* Equality of neighbors are given by the pair (addr,lladdr) */
-                siphash24_compress(&neighbor->in_addr, FAMILY_ADDRESS_SIZE(neighbor->family), state);
-                break;
-        default:
+        if (!IN_SET(neighbor->family, AF_INET, AF_INET6))
                 /* treat any other address family as AF_UNSPEC */
-                break;
-        }
+                return;
 
-        hw_addr_hash_func(&neighbor->ll_addr, state);
+        /* Equality of neighbors are given by the destination address.
+         * See neigh_lookup() in the kernel. */
+        siphash24_compress(&neighbor->in_addr, FAMILY_ADDRESS_SIZE(neighbor->family), state);
 }
 
 static int neighbor_compare_func(const Neighbor *a, const Neighbor *b) {
@@ -113,18 +108,49 @@ static int neighbor_compare_func(const Neighbor *a, const Neighbor *b) {
         if (r != 0)
                 return r;
 
-        switch (a->family) {
-        case AF_INET:
-        case AF_INET6:
-                r = memcmp(&a->in_addr, &b->in_addr, FAMILY_ADDRESS_SIZE(a->family));
-                if (r != 0)
-                        return r;
-        }
+        if (!IN_SET(a->family, AF_INET, AF_INET6))
+                /* treat any other address family as AF_UNSPEC */
+                return 0;
 
-        return hw_addr_compare(&a->ll_addr, &b->ll_addr);
+        return memcmp(&a->in_addr, &b->in_addr, FAMILY_ADDRESS_SIZE(a->family));
 }
 
-DEFINE_PRIVATE_HASH_OPS_WITH_KEY_DESTRUCTOR(neighbor_hash_ops, Neighbor, neighbor_hash_func, neighbor_compare_func, neighbor_free);
+DEFINE_PRIVATE_HASH_OPS(
+        neighbor_hash_ops,
+        Neighbor,
+        neighbor_hash_func,
+        neighbor_compare_func);
+
+DEFINE_PRIVATE_HASH_OPS_WITH_KEY_DESTRUCTOR(
+        neighbor_hash_ops_free,
+        Neighbor,
+        neighbor_hash_func,
+        neighbor_compare_func,
+        neighbor_free);
+
+static int neighbor_get_request(Link *link, const Neighbor *neighbor, Request **ret) {
+        Request *req;
+
+        assert(link);
+        assert(link->manager);
+        assert(neighbor);
+
+        req = ordered_set_get(
+                        link->manager->request_queue,
+                        &(Request) {
+                                .link = link,
+                                .type = REQUEST_TYPE_NEIGHBOR,
+                                .userdata = (void*) neighbor,
+                                .hash_func = (hash_func_t) neighbor_hash_func,
+                                .compare_func = (compare_func_t) neighbor_compare_func,
+                        });
+        if (!req)
+                return -ENOENT;
+
+        if (ret)
+                *ret = req;
+        return 0;
+}
 
 static int neighbor_get(Link *link, const Neighbor *in, Neighbor **ret) {
         Neighbor *existing;
@@ -147,7 +173,7 @@ static int neighbor_add(Link *link, Neighbor *neighbor) {
         assert(link);
         assert(neighbor);
 
-        r = set_ensure_put(&link->neighbors, &neighbor_hash_ops, neighbor);
+        r = set_ensure_put(&link->neighbors, &neighbor_hash_ops_free, neighbor);
         if (r < 0)
                 return r;
         if (r == 0)
@@ -175,24 +201,6 @@ static void log_neighbor_debug(const Neighbor *neighbor, const char *str, const
                        IN_ADDR_TO_STRING(neighbor->family, &neighbor->in_addr));
 }
 
-static int neighbor_configure_message(Neighbor *neighbor, Link *link, sd_netlink_message *req) {
-        int r;
-
-        r = sd_rtnl_message_neigh_set_state(req, NUD_PERMANENT);
-        if (r < 0)
-                return r;
-
-        r = netlink_message_append_hw_addr(req, NDA_LLADDR, &neighbor->ll_addr);
-        if (r < 0)
-                return r;
-
-        r = netlink_message_append_in_addr_union(req, NDA_DST, neighbor->family, &neighbor->in_addr);
-        if (r < 0)
-                return r;
-
-        return 0;
-}
-
 static int neighbor_configure(Neighbor *neighbor, Link *link, Request *req) {
         _cleanup_(sd_netlink_message_unrefp) sd_netlink_message *m = NULL;
         int r;
@@ -211,7 +219,15 @@ static int neighbor_configure(Neighbor *neighbor, Link *link, Request *req) {
         if (r < 0)
                 return r;
 
-        r = neighbor_configure_message(neighbor, link, m);
+        r = sd_rtnl_message_neigh_set_state(m, NUD_PERMANENT);
+        if (r < 0)
+                return r;
+
+        r = netlink_message_append_hw_addr(m, NDA_LLADDR, &neighbor->ll_addr);
+        if (r < 0)
+                return r;
+
+        r = netlink_message_append_in_addr_union(m, NDA_DST, neighbor->family, &neighbor->in_addr);
         if (r < 0)
                 return r;
 
@@ -219,6 +235,7 @@ static int neighbor_configure(Neighbor *neighbor, Link *link, Request *req) {
 }
 
 static int neighbor_process_request(Request *req, Link *link, Neighbor *neighbor) {
+        Neighbor *existing;
         int r;
 
         assert(req);
@@ -233,6 +250,9 @@ static int neighbor_process_request(Request *req, Link *link, Neighbor *neighbor
                 return log_link_warning_errno(link, r, "Failed to configure neighbor: %m");
 
         neighbor_enter_configuring(neighbor);
+        if (neighbor_get(link, neighbor, &existing) >= 0)
+                neighbor_enter_configuring(existing);
+
         return 1;
 }
 
@@ -259,31 +279,36 @@ static int static_neighbor_configure_handler(sd_netlink *rtnl, sd_netlink_messag
 }
 
 static int link_request_neighbor(Link *link, const Neighbor *neighbor) {
-        Neighbor *existing;
+        _cleanup_(neighbor_freep) Neighbor *tmp = NULL;
+        Neighbor *existing = NULL;
         int r;
 
         assert(link);
         assert(neighbor);
         assert(neighbor->source != NETWORK_CONFIG_SOURCE_FOREIGN);
 
-        if (neighbor_get(link, neighbor, &existing) < 0) {
-                _cleanup_(neighbor_freep) Neighbor *tmp = NULL;
+        if (neighbor->ll_addr.length != link->hw_addr.length) {
+                log_link_debug(link,
+                               "The link layer address length (%zu) for neighbor %s does not match with "
+                               "the hardware address length (%zu), ignoring the setting.",
+                               neighbor->ll_addr.length,
+                               IN_ADDR_TO_STRING(neighbor->family, &neighbor->in_addr),
+                               link->hw_addr.length);
+                return 0;
+        }
 
-                r = neighbor_dup(neighbor, &tmp);
-                if (r < 0)
-                        return r;
+        r = neighbor_dup(neighbor, &tmp);
+        if (r < 0)
+                return r;
 
-                r = neighbor_add(link, tmp);
-                if (r < 0)
-                        return r;
+        if (neighbor_get(link, neighbor, &existing) >= 0)
+                /* Copy state for logging below. */
+                tmp->state = existing->state;
 
-                existing = TAKE_PTR(tmp);
-        } else
-                existing->source = neighbor->source;
-
-        log_neighbor_debug(existing, "Requesting", link);
+        log_neighbor_debug(tmp, "Requesting", link);
         r = link_queue_request_safe(link, REQUEST_TYPE_NEIGHBOR,
-                                    existing, NULL,
+                                    tmp,
+                                    neighbor_free,
                                     neighbor_hash_func,
                                     neighbor_compare_func,
                                     neighbor_process_request,
@@ -293,7 +318,11 @@ static int link_request_neighbor(Link *link, const Neighbor *neighbor) {
         if (r <= 0)
                 return r;
 
-        neighbor_enter_requesting(existing);
+        neighbor_enter_requesting(tmp);
+        if (existing)
+                neighbor_enter_requesting(existing);
+
+        TAKE_PTR(tmp);
         return 1;
 }
 
@@ -307,7 +336,7 @@ int link_request_static_neighbors(Link *link) {
 
         link->static_neighbors_configured = false;
 
-        HASHMAP_FOREACH(neighbor, link->network->neighbors_by_section) {
+        ORDERED_HASHMAP_FOREACH(neighbor, link->network->neighbors_by_section) {
                 r = link_request_neighbor(link, neighbor);
                 if (r < 0)
                         return log_link_warning_errno(link, r, "Could not request neighbor: %m");
@@ -342,7 +371,8 @@ static int neighbor_remove_handler(sd_netlink *rtnl, sd_netlink_message *m, Link
 }
 
 static int neighbor_remove(Neighbor *neighbor) {
-        _cleanup_(sd_netlink_message_unrefp) sd_netlink_message *req = NULL;
+        _cleanup_(sd_netlink_message_unrefp) sd_netlink_message *m = NULL;
+        Request *req;
         Link *link;
         int r;
 
@@ -355,16 +385,16 @@ static int neighbor_remove(Neighbor *neighbor) {
 
         log_neighbor_debug(neighbor, "Removing", link);
 
-        r = sd_rtnl_message_new_neigh(link->manager->rtnl, &req, RTM_DELNEIGH,
+        r = sd_rtnl_message_new_neigh(link->manager->rtnl, &m, RTM_DELNEIGH,
                                       link->ifindex, neighbor->family);
         if (r < 0)
                 return log_link_error_errno(link, r, "Could not allocate RTM_DELNEIGH message: %m");
 
-        r = netlink_message_append_in_addr_union(req, NDA_DST, neighbor->family, &neighbor->in_addr);
+        r = netlink_message_append_in_addr_union(m, NDA_DST, neighbor->family, &neighbor->in_addr);
         if (r < 0)
                 return log_link_error_errno(link, r, "Could not append NDA_DST attribute: %m");
 
-        r = netlink_call_async(link->manager->rtnl, NULL, req, neighbor_remove_handler,
+        r = netlink_call_async(link->manager->rtnl, NULL, m, neighbor_remove_handler,
                                link_netlink_destroy_callback, link);
         if (r < 0)
                 return log_link_error_errno(link, r, "Could not send rtnetlink message: %m");
@@ -372,12 +402,15 @@ static int neighbor_remove(Neighbor *neighbor) {
         link_ref(link);
 
         neighbor_enter_removing(neighbor);
+        if (neighbor_get_request(neighbor->link, neighbor, &req) >= 0)
+                neighbor_enter_removing(req->userdata);
+
         return 0;
 }
 
 int link_drop_foreign_neighbors(Link *link) {
         Neighbor *neighbor;
-        int k, r = 0;
+        int r = 0;
 
         assert(link);
         assert(link->network);
@@ -396,7 +429,7 @@ int link_drop_foreign_neighbors(Link *link) {
         }
 
         /* Next, unmark requested neighbors. They will be configured later. */
-        HASHMAP_FOREACH(neighbor, link->network->neighbors_by_section) {
+        ORDERED_HASHMAP_FOREACH(neighbor, link->network->neighbors_by_section) {
                 Neighbor *existing;
 
                 if (neighbor_get(link, neighbor, &existing) >= 0)
@@ -407,9 +440,7 @@ int link_drop_foreign_neighbors(Link *link) {
                 if (!neighbor_is_marked(neighbor))
                         continue;
 
-                k = neighbor_remove(neighbor);
-                if (k < 0 && r >= 0)
-                        r = k;
+                RET_GATHER(r, neighbor_remove(neighbor));
         }
 
         return r;
@@ -417,7 +448,7 @@ int link_drop_foreign_neighbors(Link *link) {
 
 int link_drop_managed_neighbors(Link *link) {
         Neighbor *neighbor;
-        int k, r = 0;
+        int r = 0;
 
         assert(link);
 
@@ -430,9 +461,7 @@ int link_drop_managed_neighbors(Link *link) {
                 if (!neighbor_exists(neighbor))
                         continue;
 
-                k = neighbor_remove(neighbor);
-                if (k < 0 && r >= 0)
-                        r = k;
+                RET_GATHER(r, neighbor_remove(neighbor));
         }
 
         return r;
@@ -450,7 +479,9 @@ void link_foreignize_neighbors(Link *link) {
 int manager_rtnl_process_neighbor(sd_netlink *rtnl, sd_netlink_message *message, Manager *m) {
         _cleanup_(neighbor_freep) Neighbor *tmp = NULL;
         Neighbor *neighbor = NULL;
+        Request *req = NULL;
         uint16_t type, state;
+        bool is_new = false;
         int ifindex, r;
         Link *link;
 
@@ -494,7 +525,7 @@ int manager_rtnl_process_neighbor(sd_netlink *rtnl, sd_netlink_message *message,
         }
 
         r = link_get_by_index(m, ifindex, &link);
-        if (r < 0 || !link) {
+        if (r < 0) {
                 /* when enumerating we might be out of sync, but we will get the neighbor again. Also,
                  * kernel sends messages about neighbors after a link is removed. So, just ignore it. */
                 log_debug("rtnl: received neighbor for link '%d' we don't know about, ignoring.", ifindex);
@@ -505,6 +536,7 @@ int manager_rtnl_process_neighbor(sd_netlink *rtnl, sd_netlink_message *message,
         if (!tmp)
                 return log_oom();
 
+        /* First, retrieve the fundamental information about the neighbor. */
         r = sd_rtnl_message_neigh_get_family(message, &tmp->family);
         if (r < 0) {
                 log_link_warning(link, "rtnl: received neighbor message without family, ignoring.");
@@ -520,49 +552,52 @@ int manager_rtnl_process_neighbor(sd_netlink *rtnl, sd_netlink_message *message,
                 return 0;
         }
 
-        r = netlink_message_read_hw_addr(message, NDA_LLADDR, &tmp->ll_addr);
-        if (r < 0) {
-                log_link_warning_errno(link, r, "rtnl: received neighbor message without valid link layer address, ignoring: %m");
-                return 0;
-        }
-
+        /* Then, find the managed Neighbor and Request objects corresponding to the netlink notification. */
         (void) neighbor_get(link, tmp, &neighbor);
+        (void) neighbor_get_request(link, tmp, &req);
 
-        switch (type) {
-        case RTM_NEWNEIGH:
-                if (neighbor) {
-                        neighbor_enter_configured(neighbor);
-                        log_neighbor_debug(neighbor, "Received remembered", link);
-                } else {
-                        neighbor_enter_configured(tmp);
-                        log_neighbor_debug(tmp, "Remembering", link);
-                        r = neighbor_add(link, tmp);
-                        if (r < 0) {
-                                log_link_warning_errno(link, r, "Failed to remember foreign neighbor, ignoring: %m");
-                                return 0;
-                        }
-                        TAKE_PTR(tmp);
-                }
-
-                break;
-
-        case RTM_DELNEIGH:
+        if (type == RTM_DELNEIGH) {
                 if (neighbor) {
                         neighbor_enter_removed(neighbor);
-                        if (neighbor->state == 0) {
-                                log_neighbor_debug(neighbor, "Forgetting", link);
-                                neighbor_free(neighbor);
-                        } else
-                                log_neighbor_debug(neighbor, "Removed", link);
+                        log_neighbor_debug(neighbor, "Forgetting removed", link);
+                        neighbor_free(neighbor);
                 } else
                         log_neighbor_debug(tmp, "Kernel removed unknown", link);
 
-                break;
+                if (req)
+                        neighbor_enter_removed(req->userdata);
 
-        default:
-                assert_not_reached();
+                return 0;
         }
 
+        /* If we did not know the neighbor, then save it. */
+        if (!neighbor) {
+                r = neighbor_add(link, tmp);
+                if (r < 0) {
+                        log_link_warning_errno(link, r, "Failed to save received neighbor, ignoring: %m");
+                        return 0;
+                }
+                neighbor = TAKE_PTR(tmp);
+                is_new = true;
+        }
+
+        /* Also update information that cannot be obtained through netlink notification. */
+        if (req && req->waiting_reply) {
+                Neighbor *n = ASSERT_PTR(req->userdata);
+
+                neighbor->source = n->source;
+        }
+
+        /* Then, update miscellaneous info. */
+        r = netlink_message_read_hw_addr(message, NDA_LLADDR, &neighbor->ll_addr);
+        if (r < 0 && r != -ENODATA)
+                log_link_debug_errno(link, r, "rtnl: received neighbor message without valid link layer address, ignoring: %m");
+
+        neighbor_enter_configured(neighbor);
+        if (req)
+                neighbor_enter_configured(req->userdata);
+
+        log_neighbor_debug(neighbor, is_new ? "Remembering" : "Received remembered", link);
         return 1;
 }
 
@@ -576,6 +611,13 @@ static int neighbor_section_verify(Neighbor *neighbor) {
                                          "Ignoring [Neighbor] section from line %u.",
                                          neighbor->section->filename, neighbor->section->line);
 
+        if (neighbor->family == AF_INET6 && !socket_ipv6_is_supported())
+                return log_warning_errno(SYNTHETIC_ERRNO(EINVAL),
+                                         "%s: Neighbor section with an IPv6 destination address configured, "
+                                         "but the kernel does not support IPv6. "
+                                         "Ignoring [Neighbor] section from line %u.",
+                                         neighbor->section->filename, neighbor->section->line);
+
         if (neighbor->ll_addr.length == 0)
                 return log_warning_errno(SYNTHETIC_ERRNO(EINVAL),
                                          "%s: Neighbor section without LinkLayerAddress= configured. "
@@ -585,14 +627,45 @@ static int neighbor_section_verify(Neighbor *neighbor) {
         return 0;
 }
 
-void network_drop_invalid_neighbors(Network *network) {
+int network_drop_invalid_neighbors(Network *network) {
+        _cleanup_set_free_ Set *neighbors = NULL;
         Neighbor *neighbor;
+        int r;
 
         assert(network);
 
-        HASHMAP_FOREACH(neighbor, network->neighbors_by_section)
-                if (neighbor_section_verify(neighbor) < 0)
+        ORDERED_HASHMAP_FOREACH(neighbor, network->neighbors_by_section) {
+                Neighbor *dup;
+
+                if (neighbor_section_verify(neighbor) < 0) {
+                        /* Drop invalid [Neighbor] sections. Note that neighbor_free() will drop the
+                         * neighbor from neighbors_by_section. */
                         neighbor_free(neighbor);
+                        continue;
+                }
+
+                /* Always use the setting specified later. So, remove the previously assigned setting. */
+                dup = set_remove(neighbors, neighbor);
+                if (dup) {
+                        log_warning("%s: Duplicated neighbor settings for %s is specified at line %u and %u, "
+                                    "dropping the address setting specified at line %u.",
+                                    dup->section->filename,
+                                    IN_ADDR_TO_STRING(neighbor->family, &neighbor->in_addr),
+                                    neighbor->section->line,
+                                    dup->section->line, dup->section->line);
+                        /* neighbor_free() will drop the address from neighbors_by_section. */
+                        neighbor_free(dup);
+                }
+
+                /* Use neighbor_hash_ops, instead of neighbor_hash_ops_free. Otherwise, the Neighbor objects
+                 * will be freed. */
+                r = set_ensure_put(&neighbors, &neighbor_hash_ops, neighbor);
+                if (r < 0)
+                        return log_oom();
+                assert(r > 0);
+        }
+
+        return 0;
 }
 
 
