@@ -7,6 +7,7 @@
 #include "fd-util.h"
 #include "fileio.h"
 #include "format-util.h"
+#include "memstream-util.h"
 #include "oomd-util.h"
 #include "parse-util.h"
 #include "path-util.h"
@@ -25,8 +26,8 @@ DEFINE_HASH_OPS_WITH_VALUE_DESTRUCTOR(
                 OomdCGroupContext,
                 oomd_cgroup_context_free);
 
-static int log_kill(pid_t pid, int sig, void *userdata) {
-        log_debug("oomd attempting to kill " PID_FMT " with %s", pid, signal_to_string(sig));
+static int log_kill(const PidRef *pid, int sig, void *userdata) {
+        log_debug("oomd attempting to kill " PID_FMT " with %s", pid->pid, signal_to_string(sig));
         return 0;
 }
 
@@ -39,7 +40,7 @@ static int increment_oomd_xattr(const char *path, const char *xattr, uint64_t nu
         assert(path);
         assert(xattr);
 
-        r = cg_get_xattr_malloc(SYSTEMD_CGROUP_CONTROLLER, path, xattr, &value);
+        r = cg_get_xattr_malloc(path, xattr, &value);
         if (r < 0 && !ERRNO_IS_XATTR_ABSENT(r))
                 return r;
 
@@ -53,7 +54,7 @@ static int increment_oomd_xattr(const char *path, const char *xattr, uint64_t nu
                 return -EOVERFLOW;
 
         xsprintf(buf, "%"PRIu64, curr_count + num_procs_killed);
-        r = cg_set_xattr(SYSTEMD_CGROUP_CONTROLLER, path, xattr, buf, strlen(buf), 0);
+        r = cg_set_xattr(path, xattr, buf, strlen(buf), 0);
         if (r < 0)
                 return r;
 
@@ -156,14 +157,14 @@ int oomd_fetch_cgroup_oom_preference(OomdCGroupContext *ctx, const char *prefix)
                 return log_debug_errno(SYNTHETIC_ERRNO(EINVAL),
                                        "%s is not a descendant of %s", ctx->path, prefix);
 
-        r = cg_get_owner(SYSTEMD_CGROUP_CONTROLLER, ctx->path, &uid);
+        r = cg_get_owner(ctx->path, &uid);
         if (r < 0)
                 return log_debug_errno(r, "Failed to get owner/group from %s: %m", ctx->path);
 
         if (uid != 0) {
                 uid_t prefix_uid;
 
-                r = cg_get_owner(SYSTEMD_CGROUP_CONTROLLER, prefix, &prefix_uid);
+                r = cg_get_owner(prefix, &prefix_uid);
                 if (r < 0)
                         return log_debug_errno(r, "Failed to get owner/group from %s: %m", prefix);
 
@@ -175,14 +176,14 @@ int oomd_fetch_cgroup_oom_preference(OomdCGroupContext *ctx, const char *prefix)
 
         /* Ignore most errors when reading the xattr since it is usually unset and cgroup xattrs are only used
          * as an optional feature of systemd-oomd (and the system might not even support them). */
-        r = cg_get_xattr_bool(SYSTEMD_CGROUP_CONTROLLER, ctx->path, "user.oomd_avoid");
+        r = cg_get_xattr_bool(ctx->path, "user.oomd_avoid");
         if (r == -ENOMEM)
                 return log_oom_debug();
         if (r < 0 && !ERRNO_IS_XATTR_ABSENT(r))
                 log_debug_errno(r, "Failed to get xattr user.oomd_avoid, ignoring: %m");
         ctx->preference = r > 0 ? MANAGED_OOM_PREFERENCE_AVOID : ctx->preference;
 
-        r = cg_get_xattr_bool(SYSTEMD_CGROUP_CONTROLLER, ctx->path, "user.oomd_omit");
+        r = cg_get_xattr_bool(ctx->path, "user.oomd_omit");
         if (r == -ENOMEM)
                 return log_oom_debug();
         if (r < 0 && !ERRNO_IS_XATTR_ABSENT(r))
@@ -255,9 +256,9 @@ int oomd_cgroup_kill(const char *path, bool recurse, bool dry_run) {
                 log_debug_errno(r, "Failed to set user.oomd_ooms before kill: %m");
 
         if (recurse)
-                r = cg_kill_recursive(SYSTEMD_CGROUP_CONTROLLER, path, SIGKILL, CGROUP_IGNORE_SELF, pids_killed, log_kill, NULL);
+                r = cg_kill_recursive(path, SIGKILL, CGROUP_IGNORE_SELF, pids_killed, log_kill, NULL);
         else
-                r = cg_kill(SYSTEMD_CGROUP_CONTROLLER, path, SIGKILL, CGROUP_IGNORE_SELF, pids_killed, log_kill, NULL);
+                r = cg_kill(path, SIGKILL, CGROUP_IGNORE_SELF, pids_killed, log_kill, NULL);
 
         /* The cgroup could have been cleaned up after we have sent SIGKILL to all of the processes, but before
          * we could do one last iteration of cgroup.procs to check. Or the service unit could have exited and
@@ -281,25 +282,20 @@ int oomd_cgroup_kill(const char *path, bool recurse, bool dry_run) {
 typedef void (*dump_candidate_func)(const OomdCGroupContext *ctx, FILE *f, const char *prefix);
 
 static int dump_kill_candidates(OomdCGroupContext **sorted, int n, int dump_until, dump_candidate_func dump_func) {
-        /* Try dumping top offendors, ignoring any errors that might happen. */
-        _cleanup_free_ char *dump = NULL;
-        _cleanup_fclose_ FILE *f = NULL;
-        int r;
-        size_t size;
+        _cleanup_(memstream_done) MemStream m = {};
+        FILE *f;
 
-        f = open_memstream_unlocked(&dump, &size);
+        /* Try dumping top offendors, ignoring any errors that might happen. */
+
+        f = memstream_init(&m);
         if (!f)
-                return -errno;
+                return -ENOMEM;
 
         fprintf(f, "Considered %d cgroups for killing, top candidates were:\n", n);
         for (int i = 0; i < dump_until; i++)
                 dump_func(sorted[i], f, "\t");
 
-        r = fflush_and_check(f);
-        if (r < 0)
-                return r;
-
-        return log_dump(LOG_INFO, dump);
+        return memstream_dump(LOG_INFO, &m);
 }
 
 int oomd_kill_by_pgscan_rate(Hashmap *h, const char *prefix, bool dry_run, char **ret_selected) {
