@@ -13,7 +13,6 @@
 #include "missing_capability.h"
 #include "resolved-bus.h"
 #include "resolved-def.h"
-#include "resolved-dns-stream.h"
 #include "resolved-dns-synthesize.h"
 #include "resolved-dnssd-bus.h"
 #include "resolved-dnssd.h"
@@ -1713,11 +1712,7 @@ static int bus_method_reset_statistics(sd_bus_message *message, void *userdata,
 
         bus_client_log(message, "statistics reset");
 
-        LIST_FOREACH(scopes, s, m->dns_scopes)
-                s->cache.n_hit = s->cache.n_miss = 0;
-
-        m->n_transactions_total = 0;
-        zero(m->n_dnssec_verdict);
+        dns_manager_reset_statistics(m);
 
         return sd_bus_reply_method_return(message, NULL);
 }
@@ -1837,7 +1832,6 @@ static int bus_method_reset_server_features(sd_bus_message *message, void *userd
 
         bus_client_log(message, "server feature reset");
 
-        (void) dns_stream_disconnect_all(m);
         manager_reset_server_features(m);
 
         return sd_bus_reply_method_return(message, NULL);
@@ -2224,15 +2218,9 @@ static int match_prepare_for_sleep(sd_bus_message *message, void *userdata, sd_b
         if (b)
                 return 0;
 
-        log_debug("Coming back from suspend, closing all TCP connections...");
-        (void) dns_stream_disconnect_all(m);
-
-        log_debug("Coming back from suspend, resetting all probed server features...");
-        manager_reset_server_features(m);
-
         log_debug("Coming back from suspend, verifying all RRs...");
-        manager_verify_all(m);
 
+        manager_verify_all(m);
         return 0;
 }
 
