@@ -181,23 +181,6 @@ bool dns_resource_key_is_dnssd_ptr(const DnsResourceKey *key) {
                 dns_name_endswith(dns_resource_key_name(key), "_udp.local");
 }
 
-bool dns_resource_key_is_dnssd_two_label_ptr(const DnsResourceKey *key) {
-        assert(key);
-
-        /* Check if this is a PTR resource key used in Service Instance
-         * Enumeration as described in RFC6763 § 4.1, excluding selective
-         * service names described in RFC6763 § 7.1. */
-
-        if (key->type != DNS_TYPE_PTR)
-                return false;
-
-        const char *name = dns_resource_key_name(key);
-        if (dns_name_parent(&name) <= 0)
-                return false;
-
-        return dns_name_equal(name, "_tcp.local") || dns_name_equal(name, "_udp.local");
-}
-
 int dns_resource_key_equal(const DnsResourceKey *a, const DnsResourceKey *b) {
         int r;
 
@@ -1877,16 +1860,16 @@ int dns_resource_key_from_json(JsonVariant *v, DnsResourceKey **ret) {
         int r;
 
         JsonDispatch dispatch_table[] = {
-                { "class", JSON_VARIANT_INTEGER, json_dispatch_uint16,       PTR_TO_SIZE(&class), JSON_MANDATORY },
-                { "type",  JSON_VARIANT_INTEGER, json_dispatch_uint16,       PTR_TO_SIZE(&type),  JSON_MANDATORY },
-                { "name",  JSON_VARIANT_STRING,  json_dispatch_const_string, PTR_TO_SIZE(&name),  JSON_MANDATORY },
+                { "class", _JSON_VARIANT_TYPE_INVALID, json_dispatch_uint16,       PTR_TO_SIZE(&class), JSON_MANDATORY },
+                { "type",  _JSON_VARIANT_TYPE_INVALID, json_dispatch_uint16,       PTR_TO_SIZE(&type),  JSON_MANDATORY },
+                { "name",  JSON_VARIANT_STRING,        json_dispatch_const_string, PTR_TO_SIZE(&name),  JSON_MANDATORY },
                 {}
         };
 
         assert(v);
         assert(ret);
 
-        r = json_dispatch(v, dispatch_table, NULL, 0, NULL);
+        r = json_dispatch(v, dispatch_table, 0, NULL);
         if (r < 0)
                 return r;
 
