@@ -11,6 +11,7 @@
 #include "alloc-util.h"
 #include "architecture.h"
 #include "base-filesystem.h"
+#include "errno-util.h"
 #include "fd-util.h"
 #include "log.h"
 #include "macro.h"
@@ -36,22 +37,28 @@ static const BaseFilesystem table[] = {
         { "usr",   0755, NULL,                         NULL },
         { "var",   0755, NULL,                         NULL },
         { "etc",   0755, NULL,                         NULL },
-        { "proc",  0755, NULL,                         NULL, true },
-        { "sys",   0755, NULL,                         NULL, true },
-        { "dev",   0755, NULL,                         NULL, true },
+        { "proc",  0555, NULL,                         NULL, true },
+        { "sys",   0555, NULL,                         NULL, true },
+        { "dev",   0555, NULL,                         NULL, true },
+        { "run",   0555, NULL,                         NULL, true },
+        /* We don't add /tmp/ here for now (even though it's necessary for regular operation), because we
+         * want to support both cases where /tmp/ is a mount of its own (in which case we probably should set
+         * the mode to 1555, to indicate that no one should write to it, not even root) and when it's part of
+         * the rootfs (in which case we should set mode 1777), and we simply don't know what's right. */
 
         /* Various architecture ABIs define the path to the dynamic loader via the /lib64/ subdirectory of
          * the root directory. When booting from an otherwise empty root file system (where only /usr/ has
          * been mounted into) it is thus necessary to create a symlink pointing to the right subdirectory of
          * /usr/ first — otherwise we couldn't invoke any dynamic binary. Let's detect this case here, and
          * create the symlink as needed should it be missing. We prefer doing this consistently with Debian's
-         * multiarch logic, but support Fedora-style multilib too. */
+         * multiarch logic, but support Fedora-style and Arch-style multilib too. */
 #if defined(__aarch64__)
         /* aarch64 ELF ABI actually says dynamic loader is in /lib/, but Fedora puts it in /lib64/ anyway and
          * just symlinks /lib/ld-linux-aarch64.so.1 to ../lib64/ld-linux-aarch64.so.1. For this to work
          * correctly, /lib64/ must be symlinked to /usr/lib64/. */
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-aarch64.so.1" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-aarch64.so.1" },
 #  define KNOW_LIB64_DIRS 1
 #elif defined(__alpha__)
 #elif defined(__arc__) || defined(__tilegx__)
@@ -60,20 +67,24 @@ static const BaseFilesystem table[] = {
 #  define KNOW_LIB64_DIRS 1
 #elif defined(__i386__) || defined(__x86_64__)
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-x86-64.so.2" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-x86-64.so.2" },
 #  define KNOW_LIB64_DIRS 1
 #elif defined(__ia64__)
-#elif defined(__loongarch64)
+#elif defined(__loongarch_lp64)
 #  define KNOW_LIB64_DIRS 1
 #  if defined(__loongarch_double_float)
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-loongarch-lp64d.so.1" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-loongarch-lp64d.so.1" },
 #  elif defined(__loongarch_single_float)
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-loongarch-lp64f.so.1" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-loongarch-lp64f.so.1" },
 #  elif defined(__loongarch_soft_float)
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-loongarch-lp64s.so.1" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-loongarch-lp64s.so.1" },
 #  else
 #    error "Unknown LoongArch ABI"
 #  endif
@@ -90,7 +101,8 @@ static const BaseFilesystem table[] = {
 #elif defined(__powerpc__)
 #  if defined(__PPC64__) && __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld64.so.2" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld64.so.2" },
 #    define KNOW_LIB64_DIRS 1
 #  elif defined(__powerpc64__) && __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
         /* powerpc64-linux-gnu */
@@ -102,7 +114,8 @@ static const BaseFilesystem table[] = {
 #  elif __riscv_xlen == 64
         /* Same situation as for aarch64 */
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-linux-riscv64-lp64d.so.1" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-linux-riscv64-lp64d.so.1" },
 #    define KNOW_LIB64_DIRS 1
 #  else
 #    error "Unknown RISC-V ABI"
@@ -111,7 +124,8 @@ static const BaseFilesystem table[] = {
         /* s390-linux-gnu */
 #elif defined(__s390x__)
         { "lib64",    0, "usr/lib/"LIB_ARCH_TUPLE"\0"
-                         "usr/lib64\0",                "ld-lsb-s390x.so.3" },
+                         "usr/lib64\0"
+                         "usr/lib\0",                "ld-lsb-s390x.so.3" },
 #    define KNOW_LIB64_DIRS 1
 #elif defined(__sparc__)
 #endif
@@ -122,19 +136,19 @@ static const BaseFilesystem table[] = {
 #  pragma message "Please add an entry above specifying whether your architecture uses /lib64/, /lib32/, or no such links."
 #endif
 
-int base_filesystem_create(const char *root, uid_t uid, gid_t gid) {
-        _cleanup_close_ int fd = -EBADF;
+int base_filesystem_create_fd(int fd, const char *root, uid_t uid, gid_t gid) {
         int r;
 
-        fd = open(root, O_RDONLY|O_NONBLOCK|O_DIRECTORY|O_CLOEXEC|O_NOFOLLOW);
-        if (fd < 0)
-                return log_error_errno(errno, "Failed to open root file system: %m");
+        assert(fd >= 0);
+        assert(root);
+
+        /* The "root" parameter is decoration only – it's only used as part of log messages */
 
         for (size_t i = 0; i < ELEMENTSOF(table); i++) {
                 if (faccessat(fd, table[i].dir, F_OK, AT_SYMLINK_NOFOLLOW) >= 0)
                         continue;
 
-                if (table[i].target) {
+                if (table[i].target) { /* Create as symlink? */
                         const char *target = NULL;
 
                         /* check if one of the targets exists */
@@ -161,39 +175,36 @@ int base_filesystem_create(const char *root, uid_t uid, gid_t gid) {
                         if (!target)
                                 continue;
 
-                        if (symlinkat(target, fd, table[i].dir) < 0) {
-                                log_full_errno(IN_SET(errno, EEXIST, EROFS) || table[i].ignore_failure ? LOG_DEBUG : LOG_ERR, errno,
-                                               "Failed to create symlink at %s/%s: %m", root, table[i].dir);
-
-                                if (IN_SET(errno, EEXIST, EROFS) || table[i].ignore_failure)
-                                        continue;
-
-                                return -errno;
-                        }
-
-                        if (uid_is_valid(uid) || gid_is_valid(gid))
-                                if (fchownat(fd, table[i].dir, uid, gid, AT_SYMLINK_NOFOLLOW) < 0)
-                                        return log_error_errno(errno, "Failed to chown symlink at %s/%s: %m", root, table[i].dir);
-
-                        continue;
+                        r = RET_NERRNO(symlinkat(target, fd, table[i].dir));
+                } else {
+                        /* Create as directory. */
+                        WITH_UMASK(0000)
+                                r = RET_NERRNO(mkdirat(fd, table[i].dir, table[i].mode));
                 }
-
-                WITH_UMASK(0000)
-                        r = mkdirat(fd, table[i].dir, table[i].mode);
                 if (r < 0) {
-                        log_full_errno(IN_SET(errno, EEXIST, EROFS) || table[i].ignore_failure ? LOG_DEBUG : LOG_ERR, errno,
-                                       "Failed to create directory at %s/%s: %m", root, table[i].dir);
-
-                        if (IN_SET(errno, EEXIST, EROFS) || table[i].ignore_failure)
+                        bool ignore = IN_SET(r, -EEXIST, -EROFS) || table[i].ignore_failure;
+                        log_full_errno(ignore ? LOG_DEBUG : LOG_ERR, r,
+                                       "Failed to create %s/%s: %m", root, table[i].dir);
+                        if (ignore)
                                 continue;
 
-                        return -errno;
+                        return r;
                 }
 
                 if (uid_is_valid(uid) || gid_is_valid(gid))
                         if (fchownat(fd, table[i].dir, uid, gid, AT_SYMLINK_NOFOLLOW) < 0)
-                                return log_error_errno(errno, "Failed to chown directory at %s/%s: %m", root, table[i].dir);
+                                return log_error_errno(errno, "Failed to chown %s/%s: %m", root, table[i].dir);
         }
 
         return 0;
 }
+
+int base_filesystem_create(const char *root, uid_t uid, gid_t gid) {
+        _cleanup_close_ int fd = -EBADF;
+
+        fd = open(ASSERT_PTR(root), O_DIRECTORY|O_CLOEXEC);
+        if (fd < 0)
+                return log_error_errno(errno, "Failed to open root file system: %m");
+
+        return base_filesystem_create_fd(fd, root, uid, gid);
+}
