@@ -20,7 +20,7 @@ static inline int dlopen_idn(void) {
 
 #if HAVE_LIBIDN2
 extern int (*sym_idn2_lookup_u8)(const uint8_t* src, uint8_t** lookupname, int flags);
-extern const char *(*sym_idn2_strerror)(int rc);
+extern const char *(*sym_idn2_strerror)(int rc) _const_;
 extern int (*sym_idn2_to_unicode_8z8z)(const char * input, char ** output, int flags);
 #endif
 
