@@ -64,7 +64,7 @@ static int keymap_recurse_dir_callback(
 }
 
 int get_keymaps(char ***ret) {
-        _cleanup_(set_free_freep) Set *keymaps = NULL;
+        _cleanup_set_free_free_ Set *keymaps = NULL;
         int r;
 
         keymaps = set_new(&string_hash_ops);
@@ -82,14 +82,12 @@ int get_keymaps(char ***ret) {
                                 &(struct recurse_dir_userdata) {
                                         .keymaps = keymaps,
                                 });
-                if (r < 0) {
-                        if (r == -ENOENT)
-                                continue;
-                        if (ERRNO_IS_RESOURCE(r))
-                                return log_warning_errno(r, "Failed to read keymap list from %s: %m", dir);
-
+                if (r == -ENOENT)
+                        continue;
+                if (ERRNO_IS_NEG_RESOURCE(r))
+                        return log_warning_errno(r, "Failed to read keymap list from %s: %m", dir);
+                if (r < 0)
                         log_debug_errno(r, "Failed to read keymap list from %s, ignoring: %m", dir);
-                }
         }
 
         _cleanup_strv_free_ char **l = set_get_strv(keymaps);
@@ -129,7 +127,7 @@ bool keymap_is_valid(const char *name) {
 }
 
 int keymap_exists(const char *name) {
-        int r = 0;
+        int r;
 
         if (!keymap_is_valid(name))
                 return -EINVAL;
@@ -145,17 +143,13 @@ int keymap_exists(const char *name) {
                                 &(struct recurse_dir_userdata) {
                                         .keymap_name = name,
                                 });
-                if (r == -ENOENT)
-                        continue;
-                if (ERRNO_IS_RESOURCE(r))
-                        return r;
-                if (r < 0) {
-                        log_debug_errno(r, "Failed to read keymap list from %s, ignoring: %m", dir);
-                        continue;
-                }
                 if (r > 0)
-                        break;
+                        return true;
+                if (ERRNO_IS_NEG_RESOURCE(r))
+                        return r;
+                if (r < 0 && r != -ENOENT)
+                        log_debug_errno(r, "Failed to read keymap list from %s, ignoring: %m", dir);
         }
 
-        return r > 0;
+        return false;
 }
