@@ -27,7 +27,7 @@ int query_volatile_mode(VolatileMode *ret) {
 
                 m = volatile_mode_from_string(mode);
                 if (m < 0)
-                        return -EINVAL;
+                        return m;
 
                 *ret = m;
         } else
