@@ -17,7 +17,6 @@
 #include "io-util.h"
 #include "log.h"
 #include "main-func.h"
-#include "version.h"
 
 #define DEFAULT_BUS_PATH "unix:path=/run/dbus/system_bus_socket"
 
@@ -183,15 +182,13 @@ static int run(int argc, char *argv[]) {
                 assert_cc(sizeof(usec_t) == sizeof(uint64_t));
 
                 r = sd_bus_process(a, &m);
-                if (r < 0) {
-                        if (ERRNO_IS_DISCONNECT(r)) /* Treat 'connection reset by peer' as clean exit condition */
-                                break;
-
+                if (ERRNO_IS_NEG_DISCONNECT(r)) /* Treat 'connection reset by peer' as clean exit condition */
+                        return 0;
+                if (r < 0)
                         return log_error_errno(r, "Failed to process bus a: %m");
-                }
                 if (m) {
                         if (sd_bus_message_is_signal(m, "org.freedesktop.DBus.Local", "Disconnected"))
-                                break;
+                                return 0;
 
                         r = sd_bus_send(b, m, NULL);
                         if (r < 0)
@@ -202,16 +199,13 @@ static int run(int argc, char *argv[]) {
                         continue;
 
                 r = sd_bus_process(b, &m);
-                if (r < 0) {
-                        /* treat 'connection reset by peer' as clean exit condition */
-                        if (ERRNO_IS_DISCONNECT(r))
-                                break;
-
+                if (ERRNO_IS_NEG_DISCONNECT(r)) /* Treat 'connection reset by peer' as clean exit condition */
+                        return 0;
+                if (r < 0)
                         return log_error_errno(r, "Failed to process bus: %m");
-                }
                 if (m) {
                         if (sd_bus_message_is_signal(m, "org.freedesktop.DBus.Local", "Disconnected"))
-                                break;
+                                return 0;
 
                         r = sd_bus_send(a, m, NULL);
                         if (r < 0)
@@ -244,20 +238,15 @@ static int run(int argc, char *argv[]) {
                 t = usec_sub_unsigned(MIN(timeout_a, timeout_b), now(CLOCK_MONOTONIC));
 
                 struct pollfd p[3] = {
-                        { .fd = fd,     .events = events_a           },
-                        { .fd = in_fd,  .events = events_b & POLLIN  },
-                        { .fd = out_fd, .events = events_b & POLLOUT },
+                        { .fd = fd,            .events = events_a           },
+                        { .fd = STDIN_FILENO,  .events = events_b & POLLIN  },
+                        { .fd = STDOUT_FILENO, .events = events_b & POLLOUT },
                 };
 
                 r = ppoll_usec(p, ELEMENTSOF(p), t);
-                if (r < 0) {
-                        if (ERRNO_IS_TRANSIENT(r)) /* don't be bothered by signals, i.e. EINTR */
-                                continue;
+                if (r < 0 && !ERRNO_IS_TRANSIENT(r))  /* don't be bothered by signals, i.e. EINTR */
                         return log_error_errno(r, "ppoll() failed: %m");
-                }
         }
-
-        return 0;
 }
 
 DEFINE_MAIN_FUNCTION(run);
