@@ -93,7 +93,7 @@ int verb_cat(int argc, char *argv[], void *userdata) {
                                 arg_runtime_scope == RUNTIME_SCOPE_SYSTEM ? "" : " --user",
                                 ansi_normal());
 
-                r = cat_files(fragment_path, dropin_paths, 0);
+                r = cat_files(fragment_path, dropin_paths, /* flags= */ CAT_FORMAT_HAS_SECTIONS);
                 if (r < 0)
                         return r;
         }
@@ -344,8 +344,8 @@ int verb_edit(int argc, char *argv[], void *userdata) {
 
         STRV_FOREACH(tmp, names) {
                 r = unit_is_masked(bus, *tmp);
-                if (r < 0 && r != -ENOENT)
-                        return log_error_errno(r, "Failed to check if unit %s is masked: %m", *tmp);
+                if (r < 0)
+                        return r;
                 if (r > 0)
                         return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Cannot edit %s: unit is masked.", *tmp);
         }
