@@ -18,87 +18,6 @@ void partition_info_destroy(PartitionInfo *p) {
         p->device = mfree(p->device);
 }
 
-static int fdisk_partition_get_attrs_as_uint64(
-                struct fdisk_partition *pa,
-                uint64_t *ret) {
-
-        uint64_t flags = 0;
-        const char *a;
-        int r;
-
-        assert(pa);
-        assert(ret);
-
-        /* Retrieve current flags as uint64_t mask */
-
-        a = fdisk_partition_get_attrs(pa);
-        if (!a) {
-                *ret = 0;
-                return 0;
-        }
-
-        for (;;) {
-                _cleanup_free_ char *word = NULL;
-
-                r = extract_first_word(&a, &word, ",", EXTRACT_DONT_COALESCE_SEPARATORS);
-                if (r < 0)
-                        return r;
-                if (r == 0)
-                        break;
-
-                if (streq(word, "RequiredPartition"))
-                        flags |= SD_GPT_FLAG_REQUIRED_PARTITION;
-                else if (streq(word, "NoBlockIOProtocol"))
-                        flags |= SD_GPT_FLAG_NO_BLOCK_IO_PROTOCOL;
-                else if (streq(word, "LegacyBIOSBootable"))
-                        flags |= SD_GPT_FLAG_LEGACY_BIOS_BOOTABLE;
-                else {
-                        const char *e;
-                        unsigned u;
-
-                        /* Drop "GUID" prefix if specified */
-                        e = startswith(word, "GUID:") ?: word;
-
-                        if (safe_atou(e, &u) < 0) {
-                                log_debug("Unknown partition flag '%s', ignoring.", word);
-                                continue;
-                        }
-
-                        if (u >= sizeof(flags)*8) { /* partition flags on GPT are 64bit. Let's ignore any further
-                                                       bits should libfdisk report them */
-                                log_debug("Partition flag above bit 63 (%s), ignoring.", word);
-                                continue;
-                        }
-
-                        flags |= UINT64_C(1) << u;
-                }
-        }
-
-        *ret = flags;
-        return 0;
-}
-
-static int fdisk_partition_set_attrs_as_uint64(
-                struct fdisk_partition *pa,
-                uint64_t flags) {
-
-        _cleanup_free_ char *attrs = NULL;
-        int r;
-
-        assert(pa);
-
-        for (unsigned i = 0; i < sizeof(flags) * 8; i++) {
-                if (!FLAGS_SET(flags, UINT64_C(1) << i))
-                        continue;
-
-                r = strextendf_with_separator(&attrs, ",", "%u", i);
-                if (r < 0)
-                        return r;
-        }
-
-        return fdisk_partition_set_attrs(pa, strempty(attrs));
-}
-
 int read_partition_info(
                 struct fdisk_context *c,
                 struct fdisk_table *t,
@@ -109,6 +28,7 @@ int read_partition_info(
         const char *label;
         struct fdisk_partition *p;
         uint64_t start, size, flags;
+        unsigned long ssz;
         sd_id128_t ptid, id;
         GptPartitionType type;
         size_t partno;
@@ -135,12 +55,13 @@ int read_partition_info(
         partno = fdisk_partition_get_partno(p);
 
         start = fdisk_partition_get_start(p);
-        assert(start <= UINT64_MAX / 512U);
-        start *= 512U;
+        ssz = fdisk_get_sector_size(c);
+        assert(start <= UINT64_MAX / ssz);
+        start *= ssz;
 
         size = fdisk_partition_get_size(p);
-        assert(size <= UINT64_MAX / 512U);
-        size *= 512U;
+        assert(size <= UINT64_MAX / ssz);
+        size *= ssz;
 
         label = fdisk_partition_get_name(p);
         if (!label)
@@ -200,13 +121,9 @@ int find_suitable_partition(
         assert(device);
         assert(ret);
 
-        c = fdisk_new_context();
-        if (!c)
-                return log_oom();
-
-        r = fdisk_assign_device(c, device, /* readonly= */ true);
+        r = fdisk_new_context_at(AT_FDCWD, device, /* read_only= */ true, /* sector_size= */ UINT32_MAX, &c);
         if (r < 0)
-                return log_error_errno(r, "Failed to open device '%s': %m", device);
+                return log_error_errno(r, "Failed to create fdisk context from '%s': %m", device);
 
         if (!fdisk_is_labeltype(c, FDISK_DISKLABEL_GPT))
                 return log_error_errno(SYNTHETIC_ERRNO(EHWPOISON), "Disk %s has no GPT disk label, not suitable.", device);
@@ -269,13 +186,9 @@ int patch_partition(
         if (change == 0) /* Nothing to do */
                 return 0;
 
-        c = fdisk_new_context();
-        if (!c)
-                return log_oom();
-
-        r = fdisk_assign_device(c, device, /* readonly= */ false);
+        r = fdisk_new_context_at(AT_FDCWD, device, /* read_only= */ false, /* sector_size= */ UINT32_MAX, &c);
         if (r < 0)
-                return log_error_errno(r, "Failed to open device '%s': %m", device);
+                return log_error_errno(r, "Failed to create fdisk context from '%s': %m", device);
 
         assert_se((fd = fdisk_get_devfd(c)) >= 0);
 
