@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "alloc-util.h"
-#include "build.h"
 #include "cgroup-util.h"
 #include "dirent-util.h"
 #include "errno-util.h"
@@ -16,7 +15,6 @@
 #include "strv.h"
 #include "tests.h"
 #include "user-util.h"
-#include "util.h"
 
 static void check_p_d_u(const char *path, int code, const char *result) {
         _cleanup_free_ char *unit = NULL;
@@ -28,7 +26,7 @@ static void check_p_d_u(const char *path, int code, const char *result) {
         assert_se(streq_ptr(unit, result));
 }
 
-static void test_path_decode_unit(void) {
+TEST(path_decode_unit) {
         check_p_d_u("getty@tty2.service", 0, "getty@tty2.service");
         check_p_d_u("getty@tty2.service/", 0, "getty@tty2.service");
         check_p_d_u("getty@tty2.service/xxx", 0, "getty@tty2.service");
@@ -50,7 +48,7 @@ static void check_p_g_u(const char *path, int code, const char *result) {
         assert_se(streq_ptr(unit, result));
 }
 
-static void test_path_get_unit(void) {
+TEST(path_get_unit) {
         check_p_g_u("/system.slice/foobar.service/sdfdsaf", 0, "foobar.service");
         check_p_g_u("/system.slice/getty@tty5.service", 0, "getty@tty5.service");
         check_p_g_u("/system.slice/getty@tty5.service/aaa/bbb", 0, "getty@tty5.service");
@@ -64,6 +62,33 @@ static void test_path_get_unit(void) {
         check_p_g_u("/user.slice/user-1000.slice/user@.service/server.service", -ENXIO, NULL);
 }
 
+static void check_p_g_u_p(const char *path, int code, const char *result) {
+        _cleanup_free_ char *unit_path = NULL;
+        int r;
+
+        r = cg_path_get_unit_path(path, &unit_path);
+        printf("%s: %s → %s %d expected %s %d\n", __func__, path, unit_path, r, strnull(result), code);
+        assert_se(r == code);
+        assert_se(streq_ptr(unit_path, result));
+}
+
+TEST(path_get_unit_path) {
+        check_p_g_u_p("/system.slice/foobar.service/sdfdsaf", 0, "/system.slice/foobar.service");
+        check_p_g_u_p("/system.slice/getty@tty5.service", 0, "/system.slice/getty@tty5.service");
+        check_p_g_u_p("/system.slice/getty@tty5.service/aaa/bbb", 0, "/system.slice/getty@tty5.service");
+        check_p_g_u_p("/system.slice/getty@tty5.service/", 0, "/system.slice/getty@tty5.service");
+        check_p_g_u_p("/system.slice/getty@tty6.service/tty5", 0, "/system.slice/getty@tty6.service");
+        check_p_g_u_p("sadfdsafsda", -ENXIO, NULL);
+        check_p_g_u_p("/system.slice/getty####@tty6.service/xxx", -ENXIO, NULL);
+        check_p_g_u_p("/system.slice/system-waldo.slice/foobar.service/sdfdsaf", 0, "/system.slice/system-waldo.slice/foobar.service");
+        check_p_g_u_p("/system.slice/system-waldo.slice/_cpu.service/sdfdsaf", 0, "/system.slice/system-waldo.slice/_cpu.service");
+        check_p_g_u_p("/system.slice/system-waldo.slice/_cpu.service", 0, "/system.slice/system-waldo.slice/_cpu.service");
+        check_p_g_u_p("/user.slice/user-1000.slice/user@1000.service/server.service", 0, "/user.slice/user-1000.slice/user@1000.service");
+        check_p_g_u_p("/user.slice/user-1000.slice/user@.service/server.service", -ENXIO, NULL);
+        check_p_g_u_p("/user.slice/_user-1000.slice/user@1000.service/foobar.slice/foobar@pie.service", 0, "/user.slice/_user-1000.slice/user@1000.service");
+        check_p_g_u_p("/_session-2.scope/_foobar@pie.service/pa/po", 0, "/_session-2.scope");
+}
+
 static void check_p_g_u_u(const char *path, int code, const char *result) {
         _cleanup_free_ char *unit = NULL;
         int r;
@@ -74,7 +99,7 @@ static void check_p_g_u_u(const char *path, int code, const char *result) {
         assert_se(streq_ptr(unit, result));
 }
 
-static void test_path_get_user_unit(void) {
+TEST(path_get_user_unit) {
         check_p_g_u_u("/user.slice/user-1000.slice/session-2.scope/foobar.service", 0, "foobar.service");
         check_p_g_u_u("/user.slice/user-1000.slice/session-2.scope/waldo.slice/foobar.service", 0, "foobar.service");
         check_p_g_u_u("/user.slice/user-1002.slice/session-2.scope/foobar.service/waldo", 0, "foobar.service");
@@ -97,7 +122,7 @@ static void check_p_g_s(const char *path, int code, const char *result) {
         assert_se(streq_ptr(s, result));
 }
 
-static void test_path_get_session(void) {
+TEST(path_get_session) {
         check_p_g_s("/user.slice/user-1000.slice/session-2.scope/foobar.service", 0, "2");
         check_p_g_s("/session-3.scope", 0, "3");
         check_p_g_s("/session-.scope", -ENXIO, NULL);
@@ -111,7 +136,7 @@ static void check_p_g_o_u(const char *path, int code, uid_t result) {
         assert_se(uid == result);
 }
 
-static void test_path_get_owner_uid(void) {
+TEST(path_get_owner_uid) {
         check_p_g_o_u("/user.slice/user-1000.slice/session-2.scope/foobar.service", 0, 1000);
         check_p_g_o_u("/user.slice/user-1006.slice", 0, 1006);
         check_p_g_o_u("", -ENXIO, 0);
@@ -124,7 +149,7 @@ static void check_p_g_slice(const char *path, int code, const char *result) {
         assert_se(streq_ptr(s, result));
 }
 
-static void test_path_get_slice(void) {
+TEST(path_get_slice) {
         check_p_g_slice("/user.slice", 0, "user.slice");
         check_p_g_slice("/foobar", 0, SPECIAL_ROOT_SLICE);
         check_p_g_slice("/user.slice/user-waldo.slice", 0, "user-waldo.slice");
@@ -141,7 +166,7 @@ static void check_p_g_u_slice(const char *path, int code, const char *result) {
         assert_se(streq_ptr(s, result));
 }
 
-static void test_path_get_user_slice(void) {
+TEST(path_get_user_slice) {
         check_p_g_u_slice("/user.slice", -ENXIO, NULL);
         check_p_g_u_slice("/foobar", -ENXIO, NULL);
         check_p_g_u_slice("/user.slice/user-waldo.slice", -ENXIO, NULL);
@@ -158,47 +183,44 @@ static void test_path_get_user_slice(void) {
         check_p_g_u_slice("/foo.slice//foo-bar.slice/user@1000.service/piep.slice//piep-pap.slice//foo.service", 0, "piep-pap.slice");
 }
 
-static void test_get_paths(void) {
+TEST(get_paths, .sd_booted = true) {
         _cleanup_free_ char *a = NULL;
 
         assert_se(cg_get_root_path(&a) >= 0);
         log_info("Root = %s", a);
 }
 
-static void test_proc(void) {
+TEST(proc) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r;
 
-        d = opendir("/proc");
-        assert_se(d);
+        assert_se(proc_dir_open(&d) >= 0);
 
-        FOREACH_DIRENT(de, d, break) {
+        for (;;) {
                 _cleanup_free_ char *path = NULL, *path_shifted = NULL, *session = NULL, *unit = NULL, *user_unit = NULL, *machine = NULL, *slice = NULL;
-                pid_t pid;
+                _cleanup_(pidref_done) PidRef pid = PIDREF_NULL;
                 uid_t uid = UID_INVALID;
 
-                if (!IN_SET(de->d_type, DT_DIR, DT_UNKNOWN))
+                r = proc_dir_read_pidref(d, &pid);
+                assert_se(r >= 0);
+
+                if (r == 0)
+                        break;
+
+                if (pidref_is_kernel_thread(&pid) != 0)
                         continue;
 
-                r = parse_pid(de->d_name, &pid);
-                if (r < 0)
-                        continue;
-
-                if (is_kernel_thread(pid))
-                        continue;
-
-                cg_pid_get_path(SYSTEMD_CGROUP_CONTROLLER, pid, &path);
-                cg_pid_get_path_shifted(pid, NULL, &path_shifted);
-                cg_pid_get_owner_uid(pid, &uid);
-                cg_pid_get_session(pid, &session);
-                cg_pid_get_unit(pid, &unit);
-                cg_pid_get_user_unit(pid, &user_unit);
-                cg_pid_get_machine_name(pid, &machine);
-                cg_pid_get_slice(pid, &slice);
+                cg_pid_get_path(SYSTEMD_CGROUP_CONTROLLER, pid.pid, &path);
+                cg_pid_get_path_shifted(pid.pid, NULL, &path_shifted);
+                cg_pid_get_owner_uid(pid.pid, &uid);
+                cg_pid_get_session(pid.pid, &session);
+                cg_pid_get_unit(pid.pid, &unit);
+                cg_pid_get_user_unit(pid.pid, &user_unit);
+                cg_pid_get_machine_name(pid.pid, &machine);
+                cg_pid_get_slice(pid.pid, &slice);
 
                 printf(PID_FMT"\t%s\t%s\t"UID_FMT"\t%s\t%s\t%s\t%s\t%s\n",
-                       pid,
+                       pid.pid,
                        path,
                        path_shifted,
                        uid,
@@ -210,17 +232,22 @@ static void test_proc(void) {
         }
 }
 
-static void test_escape_one(const char *s, const char *r) {
-        _cleanup_free_ char *b;
+static void test_escape_one(const char *s, const char *expected) {
+        _cleanup_free_ char *b = NULL;
 
-        b = cg_escape(s);
-        assert_se(b);
-        assert_se(streq(b, r));
+        assert_se(s);
+        assert_se(expected);
+
+        assert_se(cg_escape(s, &b) >= 0);
+        assert_se(streq(b, expected));
 
         assert_se(streq(cg_unescape(b), s));
+
+        assert_se(filename_is_valid(b));
+        assert_se(!cg_needs_escape(s) || b[0] == '_');
 }
 
-static void test_escape(void) {
+TEST(escape, .sd_booted = true) {
         test_escape_one("foobar", "foobar");
         test_escape_one(".foobar", "_.foobar");
         test_escape_one("foobar.service", "foobar.service");
@@ -234,7 +261,7 @@ static void test_escape(void) {
         test_escape_one(".", "_.");
 }
 
-static void test_controller_is_valid(void) {
+TEST(controller_is_valid) {
         assert_se(cg_controller_is_valid("foobar"));
         assert_se(cg_controller_is_valid("foo_bar"));
         assert_se(cg_controller_is_valid("name=foo"));
@@ -260,7 +287,7 @@ static void test_slice_to_path_one(const char *unit, const char *path, int error
         assert_se(streq_ptr(ret, path));
 }
 
-static void test_slice_to_path(void) {
+TEST(slice_to_path) {
         test_slice_to_path_one("foobar.slice", "foobar.slice", 0);
         test_slice_to_path_one("foobar-waldo.slice", "foobar.slice/foobar-waldo.slice", 0);
         test_slice_to_path_one("foobar-waldo.service", NULL, -EINVAL);
@@ -292,16 +319,14 @@ static void test_shift_path_one(const char *raw, const char *root, const char *s
         assert_se(streq(s, shifted));
 }
 
-static void test_shift_path(void) {
-
+TEST(shift_path) {
         test_shift_path_one("/foobar/waldo", "/", "/foobar/waldo");
         test_shift_path_one("/foobar/waldo", "", "/foobar/waldo");
         test_shift_path_one("/foobar/waldo", "/foobar", "/waldo");
         test_shift_path_one("/foobar/waldo", "/hogehoge", "/foobar/waldo");
 }
 
-static void test_mask_supported(void) {
-
+TEST(mask_supported, .sd_booted = true) {
         CGroupMask m;
         CGroupController c;
 
@@ -311,7 +336,7 @@ static void test_mask_supported(void) {
                 printf("'%s' is supported: %s\n", cgroup_controller_to_string(c), yes_no(m & CGROUP_CONTROLLER_TO_MASK(c)));
 }
 
-static void test_is_cgroup_fs(void) {
+TEST(is_cgroup_fs, .sd_booted = true) {
         struct statfs sfs;
         assert_se(statfs("/sys/fs/cgroup", &sfs) == 0);
         if (is_temporary_fs(&sfs))
@@ -319,7 +344,7 @@ static void test_is_cgroup_fs(void) {
         assert_se(is_cgroup_fs(&sfs));
 }
 
-static void test_fd_is_cgroup_fs(void) {
+TEST(fd_is_cgroup_fs, .sd_booted = true) {
         int fd;
 
         fd = open("/sys/fs/cgroup", O_RDONLY|O_DIRECTORY|O_CLOEXEC|O_NOFOLLOW);
@@ -333,12 +358,12 @@ static void test_fd_is_cgroup_fs(void) {
         fd = safe_close(fd);
 }
 
-static void test_cg_tests(void) {
+TEST(cg_tests) {
         int all, hybrid, systemd, r;
 
         r = cg_unified();
         if (r == -ENOMEDIUM) {
-                log_notice_errno(r, "Skipping cg hierarchy tests: %m");
+                log_tests_skipped("cgroup not mounted");
                 return;
         }
         assert_se(r >= 0);
@@ -364,7 +389,7 @@ static void test_cg_tests(void) {
                 assert_se(!systemd);
 }
 
-static void test_cg_get_keyed_attribute(void) {
+TEST(cg_get_keyed_attribute) {
         _cleanup_free_ char *val = NULL;
         char *vals3[3] = {}, *vals3a[3] = {};
         int i, r;
@@ -395,12 +420,12 @@ static void test_cg_get_keyed_attribute(void) {
 
         assert_se(cg_get_keyed_attribute("cpu", "/init.scope", "cpu.stat", STRV_MAKE("usage_usec", "no_such_attr"), vals3) == -ENXIO);
         assert_se(cg_get_keyed_attribute_graceful("cpu", "/init.scope", "cpu.stat", STRV_MAKE("usage_usec", "no_such_attr"), vals3) == 1);
-        assert(vals3[0] && !vals3[1]);
+        assert_se(vals3[0] && !vals3[1]);
         free(vals3[0]);
 
         assert_se(cg_get_keyed_attribute("cpu", "/init.scope", "cpu.stat", STRV_MAKE("usage_usec", "usage_usec"), vals3) == -ENXIO);
         assert_se(cg_get_keyed_attribute_graceful("cpu", "/init.scope", "cpu.stat", STRV_MAKE("usage_usec", "usage_usec"), vals3) == 1);
-        assert(vals3[0] && !vals3[1]);
+        assert_se(vals3[0] && !vals3[1]);
         free(vals3[0]);
 
         assert_se(cg_get_keyed_attribute("cpu", "/init.scope", "cpu.stat",
@@ -429,27 +454,13 @@ static void test_cg_get_keyed_attribute(void) {
         }
 }
 
-int main(void) {
-        test_setup_logging(LOG_DEBUG);
-
-        test_path_decode_unit();
-        test_path_get_unit();
-        test_path_get_user_unit();
-        test_path_get_session();
-        test_path_get_owner_uid();
-        test_path_get_slice();
-        test_path_get_user_slice();
-        TEST_REQ_RUNNING_SYSTEMD(test_get_paths());
-        test_proc();
-        TEST_REQ_RUNNING_SYSTEMD(test_escape());
-        test_controller_is_valid();
-        test_slice_to_path();
-        test_shift_path();
-        TEST_REQ_RUNNING_SYSTEMD(test_mask_supported());
-        TEST_REQ_RUNNING_SYSTEMD(test_is_cgroup_fs());
-        TEST_REQ_RUNNING_SYSTEMD(test_fd_is_cgroup_fs());
-        test_cg_tests();
-        test_cg_get_keyed_attribute();
-
-        return 0;
+TEST(bfq_weight_conversion) {
+        assert_se(BFQ_WEIGHT(1) == 1);
+        assert_se(BFQ_WEIGHT(50) == 50);
+        assert_se(BFQ_WEIGHT(100) == 100);
+        assert_se(BFQ_WEIGHT(500) == 136);
+        assert_se(BFQ_WEIGHT(5000) == 545);
+        assert_se(BFQ_WEIGHT(10000) == 1000);
 }
+
+DEFINE_TEST_MAIN(LOG_DEBUG);
