@@ -251,7 +251,7 @@ TEST(condition_test_host) {
         int r;
 
         r = sd_id128_get_machine(&id);
-        if (r < 0 && ERRNO_IS_MACHINE_ID_UNSET(r))
+        if (ERRNO_IS_NEG_MACHINE_ID_UNSET(r))
                 return (void) log_tests_skipped("/etc/machine-id missing");
         assert_se(r >= 0);
 
@@ -1003,13 +1003,6 @@ TEST(condition_test_group) {
         condition_free(condition);
         free(gid);
 
-        /* In an unprivileged user namespace with the current user mapped to root, all the auxiliary groups
-         * of the user will be mapped to the nobody group, which means the user in the user namespace is in
-         * both the root and the nobody group, meaning the next test can't work, so let's skip it in that
-         * case. */
-        if (in_group(NOBODY_GROUP_NAME) && in_group("root"))
-                return (void) log_tests_skipped("user is in both root and nobody group");
-
         groupname = (char*)(getegid() == 0 ? NOBODY_GROUP_NAME : "root");
         condition = condition_new(CONDITION_GROUP, groupname, false, false);
         assert_se(condition);
@@ -1246,24 +1239,6 @@ TEST(condition_test_os_release) {
         assert_se(condition_test(condition, environ) > 0);
         condition_free(condition);
 
-        /* Test shell style globs */
-
-        assert_se((condition = condition_new(CONDITION_OS_RELEASE, "ID_LIKE$=*THISHOPEFULLYWONTEXIST*", false, false)));
-        assert_se(condition_test(condition, environ) == 0);
-        condition_free(condition);
-
-        assert_se((condition = condition_new(CONDITION_OS_RELEASE, "ID_THISHOPEFULLYWONTEXIST$=*rhel*", false, false)));
-        assert_se(condition_test(condition, environ) == 0);
-        condition_free(condition);
-
-        assert_se((condition = condition_new(CONDITION_OS_RELEASE, "ID_LIKE!$=*THISHOPEFULLYWONTEXIST*", false, false)));
-        assert_se(condition_test(condition, environ) >= 0);
-        condition_free(condition);
-
-        assert_se((condition = condition_new(CONDITION_OS_RELEASE, "ID_THISHOPEFULLYWONTEXIST!$=*rhel*", false, false)));
-        assert_se(condition_test(condition, environ) >= 0);
-        condition_free(condition);
-
         /* load_os_release_pairs() removes quotes, we have to add them back,
          * otherwise we get a string: "PRETTY_NAME=Debian GNU/Linux 10 (buster)"
          * which is wrong, as the value is not quoted anymore. */
