@@ -1,9 +1,12 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include "alloc-util.h"
 #include "list.h"
-#include "util.h"
+#include "tests.h"
 
 int main(int argc, const char *argv[]) {
+        test_setup_logging(LOG_DEBUG);
+
         size_t i;
         typedef struct list_item {
                 LIST_FIELDS(struct list_item, item_list);
@@ -20,7 +23,7 @@ int main(int argc, const char *argv[]) {
         for (i = 0; i < ELEMENTSOF(items); i++) {
                 LIST_INIT(item_list, &items[i]);
                 assert_se(LIST_JUST_US(item_list, &items[i]));
-                LIST_PREPEND(item_list, head, &items[i]);
+                assert_se(LIST_PREPEND(item_list, head, &items[i]) == &items[i]);
         }
 
         i = 0;
@@ -56,14 +59,13 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        list_item *cursor;
-        LIST_FIND_HEAD(item_list, &items[0], cursor);
+        list_item *cursor = LIST_FIND_HEAD(item_list, &items[0]);
         assert_se(cursor == &items[3]);
 
-        LIST_FIND_TAIL(item_list, &items[3], cursor);
+        cursor = LIST_FIND_TAIL(item_list, &items[3]);
         assert_se(cursor == &items[0]);
 
-        LIST_REMOVE(item_list, head, &items[1]);
+        assert_se(LIST_REMOVE(item_list, head, &items[1]) == &items[1]);
         assert_se(LIST_JUST_US(item_list, &items[1]));
 
         assert_se(items[0].item_list_next == NULL);
@@ -74,7 +76,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_INSERT_AFTER(item_list, head, &items[3], &items[1]);
+        assert_se(LIST_INSERT_AFTER(item_list, head, &items[3], &items[1]) == &items[1]);
         assert_se(items[0].item_list_next == NULL);
         assert_se(items[2].item_list_next == &items[0]);
         assert_se(items[1].item_list_next == &items[2]);
@@ -85,7 +87,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_REMOVE(item_list, head, &items[1]);
+        assert_se(LIST_REMOVE(item_list, head, &items[1]) == &items[1]);
         assert_se(LIST_JUST_US(item_list, &items[1]));
 
         assert_se(items[0].item_list_next == NULL);
@@ -96,7 +98,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_INSERT_BEFORE(item_list, head, &items[2], &items[1]);
+        assert_se(LIST_INSERT_BEFORE(item_list, head, &items[2], &items[1]) == &items[1]);
         assert_se(items[0].item_list_next == NULL);
         assert_se(items[2].item_list_next == &items[0]);
         assert_se(items[1].item_list_next == &items[2]);
@@ -107,7 +109,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_REMOVE(item_list, head, &items[0]);
+        assert_se(LIST_REMOVE(item_list, head, &items[0]) == &items[0]);
         assert_se(LIST_JUST_US(item_list, &items[0]));
 
         assert_se(items[2].item_list_next == NULL);
@@ -118,7 +120,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_INSERT_BEFORE(item_list, head, &items[3], &items[0]);
+        assert_se(LIST_INSERT_BEFORE(item_list, head, &items[3], &items[0]) == &items[0]);
         assert_se(items[2].item_list_next == NULL);
         assert_se(items[1].item_list_next == &items[2]);
         assert_se(items[3].item_list_next == &items[1]);
@@ -130,7 +132,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[0].item_list_prev == NULL);
         assert_se(head == &items[0]);
 
-        LIST_REMOVE(item_list, head, &items[0]);
+        assert_se(LIST_REMOVE(item_list, head, &items[0]) == &items[0]);
         assert_se(LIST_JUST_US(item_list, &items[0]));
 
         assert_se(items[2].item_list_next == NULL);
@@ -141,7 +143,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_INSERT_BEFORE(item_list, head, NULL, &items[0]);
+        assert_se(LIST_INSERT_BEFORE(item_list, head, NULL, &items[0]) == &items[0]);
         assert_se(items[0].item_list_next == NULL);
         assert_se(items[2].item_list_next == &items[0]);
         assert_se(items[1].item_list_next == &items[2]);
@@ -152,7 +154,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_REMOVE(item_list, head, &items[0]);
+        assert_se(LIST_REMOVE(item_list, head, &items[0]) == &items[0]);
         assert_se(LIST_JUST_US(item_list, &items[0]));
 
         assert_se(items[2].item_list_next == NULL);
@@ -163,7 +165,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[1].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_REMOVE(item_list, head, &items[1]);
+        assert_se(LIST_REMOVE(item_list, head, &items[1]) == &items[1]);
         assert_se(LIST_JUST_US(item_list, &items[1]));
 
         assert_se(items[2].item_list_next == NULL);
@@ -172,18 +174,18 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_REMOVE(item_list, head, &items[2]);
+        assert_se(LIST_REMOVE(item_list, head, &items[2]) == &items[2]);
         assert_se(LIST_JUST_US(item_list, &items[2]));
         assert_se(LIST_JUST_US(item_list, head));
 
-        LIST_REMOVE(item_list, head, &items[3]);
+        assert_se(LIST_REMOVE(item_list, head, &items[3]) == &items[3]);
         assert_se(LIST_JUST_US(item_list, &items[3]));
 
         assert_se(head == NULL);
 
         for (i = 0; i < ELEMENTSOF(items); i++) {
                 assert_se(LIST_JUST_US(item_list, &items[i]));
-                LIST_APPEND(item_list, head, &items[i]);
+                assert_se(LIST_APPEND(item_list, head, &items[i]) == &items[i]);
         }
 
         assert_se(!LIST_JUST_US(item_list, head));
@@ -199,20 +201,20 @@ int main(int argc, const char *argv[]) {
         assert_se(items[3].item_list_prev == &items[2]);
 
         for (i = 0; i < ELEMENTSOF(items); i++)
-                LIST_REMOVE(item_list, head, &items[i]);
+                assert_se(LIST_REMOVE(item_list, head, &items[i]) == &items[i]);
 
         assert_se(head == NULL);
 
         for (i = 0; i < ELEMENTSOF(items) / 2; i++) {
                 LIST_INIT(item_list, &items[i]);
                 assert_se(LIST_JUST_US(item_list, &items[i]));
-                LIST_PREPEND(item_list, head, &items[i]);
+                assert_se(LIST_PREPEND(item_list, head, &items[i]) == &items[i]);
         }
 
         for (i = ELEMENTSOF(items) / 2; i < ELEMENTSOF(items); i++) {
                 LIST_INIT(item_list, &items[i]);
                 assert_se(LIST_JUST_US(item_list, &items[i]));
-                LIST_PREPEND(item_list, head2, &items[i]);
+                assert_se(LIST_PREPEND(item_list, head2, &items[i]) == &items[i]);
         }
 
         assert_se(items[0].item_list_next == NULL);
@@ -225,7 +227,7 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_JOIN(item_list, head2, head);
+        assert_se(LIST_JOIN(item_list, head2, head) == head2);
         assert_se(head == NULL);
 
         assert_se(items[0].item_list_next == NULL);
@@ -238,23 +240,47 @@ int main(int argc, const char *argv[]) {
         assert_se(items[2].item_list_prev == &items[3]);
         assert_se(items[3].item_list_prev == NULL);
 
-        LIST_JOIN(item_list, head, head2);
+        assert_se(LIST_JOIN(item_list, head, head2) == head);
         assert_se(head2 == NULL);
         assert_se(head);
 
         for (i = 0; i < ELEMENTSOF(items); i++)
-                LIST_REMOVE(item_list, head, &items[i]);
+                assert_se(LIST_REMOVE(item_list, head, &items[i]) == &items[i]);
 
         assert_se(head == NULL);
 
-        LIST_PREPEND(item_list, head, items + 0);
-        LIST_PREPEND(item_list, head, items + 1);
-        LIST_PREPEND(item_list, head, items + 2);
+        assert_se(LIST_PREPEND(item_list, head, items + 0) == items + 0);
+        assert_se(LIST_PREPEND(item_list, head, items + 1) == items + 1);
+        assert_se(LIST_PREPEND(item_list, head, items + 2) == items + 2);
 
         assert_se(LIST_POP(item_list, head) == items + 2);
         assert_se(LIST_POP(item_list, head) == items + 1);
         assert_se(LIST_POP(item_list, head) == items + 0);
         assert_se(LIST_POP(item_list, head) == NULL);
 
+        /* No-op on an empty list */
+
+        LIST_CLEAR(item_list, head, free);
+
+        /* A non-empty list is cleared */
+
+        assert_se(LIST_PREPEND(item_list, head, new0(list_item, 1)));
+        assert_se(LIST_PREPEND(item_list, head, new0(list_item, 1)));
+
+        LIST_CLEAR(item_list, head, free);
+
+        assert_se(head == NULL);
+
+        /* A list can be cleared partially */
+
+        assert_se(LIST_PREPEND(item_list, head, new0(list_item, 1)));
+        assert_se(LIST_PREPEND(item_list, head, new0(list_item, 1)));
+        assert_se(LIST_PREPEND(item_list, head, items + 0) == items + 0);
+
+        LIST_CLEAR(item_list, head->item_list_next, free);
+
+        assert_se(head == items + 0);
+        assert_se(head->item_list_next == NULL);
+
         return 0;
 }
