@@ -5,8 +5,6 @@
 #include "strv.h"
 #include "tests.h"
 
-const bool mempool_use_allowed = VALGRIND;
-
 TEST(set_steal_first) {
         _cleanup_set_free_ Set *m = NULL;
         int seen[3] = {};
@@ -37,10 +35,9 @@ static void item_seen(Item *item) {
 TEST(set_free_with_destructor) {
         Set *m;
         struct Item items[4] = {};
-        unsigned i;
 
         assert_se(m = set_new(NULL));
-        for (i = 0; i < ELEMENTSOF(items) - 1; i++)
+        for (size_t i = 0; i < ELEMENTSOF(items) - 1; i++)
                 assert_se(set_put(m, items + i) == 1);
 
         m = set_free_with_destructor(m, item_seen);
@@ -55,10 +52,9 @@ DEFINE_PRIVATE_HASH_OPS_WITH_VALUE_DESTRUCTOR(item_hash_ops, void, trivial_hash_
 TEST(set_free_with_hash_ops) {
         Set *m;
         struct Item items[4] = {};
-        unsigned i;
 
         assert_se(m = set_new(&item_hash_ops));
-        for (i = 0; i < ELEMENTSOF(items) - 1; i++)
+        for (size_t i = 0; i < ELEMENTSOF(items) - 1; i++)
                 assert_se(set_put(m, items + i) == 1);
 
         m = set_free(m);
@@ -149,7 +145,8 @@ TEST(set_ensure_allocated) {
 }
 
 TEST(set_copy) {
-        Set *s, *copy;
+        _cleanup_set_free_ Set *s = NULL;
+        _cleanup_set_free_free_ Set *copy = NULL;
         char *key1, *key2, *key3, *key4;
 
         key1 = strdup("key1");
@@ -173,9 +170,6 @@ TEST(set_copy) {
         assert_se(copy);
 
         assert_se(set_equal(s, copy));
-
-        set_free(s);
-        set_free_free(copy);
 }
 
 TEST(set_ensure_put) {
