@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 /* Some unit tests for the helper functions in timesyncd. */
 
@@ -7,7 +7,7 @@
 #include "timesyncd-conf.h"
 #include "tests.h"
 
-static void test_manager_parse_string(void) {
+TEST(manager_parse_string) {
         /* Make sure that NTP_SERVERS is configured to something
          * that we can actually parse successfully. */
 
@@ -25,10 +25,4 @@ static void test_manager_parse_string(void) {
         assert_se(manager_parse_server_string(m, SERVER_LINK, "time1.foobar.com time2.foobar.com axrfav.,avf..ra 12345..123") == 0);
 }
 
-int main(int argc, char **argv) {
-        test_setup_logging(LOG_DEBUG);
-
-        test_manager_parse_string();
-
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_DEBUG);
