@@ -3,6 +3,7 @@
 #include <getopt.h>
 #include <utmp.h>
 
+#include "build.h"
 #include "dirent-util.h"
 #include "errno-list.h"
 #include "escape.h"
@@ -22,7 +23,6 @@
 #include "user-util.h"
 #include "userdb.h"
 #include "verbs.h"
-#include "virt.h"
 
 static enum {
         OUTPUT_CLASSIC,
@@ -130,16 +130,10 @@ static int show_user(UserRecord *ur, Table *table) {
         return 0;
 }
 
-static bool test_show_mapped(void) {
-        /* Show mapped user range only in environments where user mapping is a thing. */
-        return running_in_userns() > 0;
-}
-
 static const struct {
         uid_t first, last;
         const char *name;
         UserDisposition disposition;
-        bool (*test)(void);
 } uid_range_table[] = {
         {
                 .first = 1,
@@ -172,12 +166,11 @@ static const struct {
                 .last = MAP_UID_MAX,
                 .name = "mapped",
                 .disposition = USER_REGULAR,
-                .test = test_show_mapped,
         },
 };
 
 static int table_add_uid_boundaries(Table *table, const UidRange *p) {
-        int r, n_added = 0;
+        int r;
 
         assert(table);
 
@@ -187,9 +180,6 @@ static int table_add_uid_boundaries(Table *table, const UidRange *p) {
                 if (!uid_range_covers(p, uid_range_table[i].first, uid_range_table[i].last - uid_range_table[i].first + 1))
                         continue;
 
-                if (uid_range_table[i].test && !uid_range_table[i].test())
-                        continue;
-
                 name = strjoin(special_glyph(SPECIAL_GLYPH_ARROW_DOWN),
                                " begin ", uid_range_table[i].name, " users ",
                                special_glyph(SPECIAL_GLYPH_ARROW_DOWN));
@@ -247,11 +237,9 @@ static int table_add_uid_boundaries(Table *table, const UidRange *p) {
                                 TABLE_INT, 1); /* sort after any other entry with the same UID */
                 if (r < 0)
                         return table_log_add_error(r);
-
-                n_added += 2;
         }
 
-        return n_added;
+        return ELEMENTSOF(uid_range_table) * 2;
 }
 
 static int add_unavailable_uid(Table *table, uid_t start, uid_t end) {
@@ -322,9 +310,6 @@ static int table_add_uid_map(
         assert(table);
         assert(add_unavailable);
 
-        if (!p)
-                return 0;
-
         for (size_t i = 0; p && i < p->n_entries; i++) {
                 UidRangeEntry *x = p->entries + i;
 
@@ -542,18 +527,14 @@ static int show_group(GroupRecord *gr, Table *table) {
 }
 
 static int table_add_gid_boundaries(Table *table, const UidRange *p) {
-        int r, n_added = 0;
+        int r;
 
         assert(table);
 
         for (size_t i = 0; i < ELEMENTSOF(uid_range_table); i++) {
                 _cleanup_free_ char *name = NULL, *comment = NULL;
 
-                if (!uid_range_covers(p, uid_range_table[i].first,
-                                      uid_range_table[i].last - uid_range_table[i].first + 1))
-                        continue;
-
-                if (uid_range_table[i].test && !uid_range_table[i].test())
+                if (!uid_range_covers(p, uid_range_table[i].first, uid_range_table[i].last))
                         continue;
 
                 name = strjoin(special_glyph(SPECIAL_GLYPH_ARROW_DOWN),
@@ -607,11 +588,9 @@ static int table_add_gid_boundaries(Table *table, const UidRange *p) {
                                 TABLE_INT, 1); /* sort after any other entry with the same GID */
                 if (r < 0)
                         return table_log_add_error(r);
-
-                n_added += 2;
         }
 
-        return n_added;
+        return ELEMENTSOF(uid_range_table) * 2;
 }
 
 static int add_unavailable_gid(Table *table, uid_t start, uid_t end) {
@@ -931,7 +910,7 @@ static int display_memberships(int argc, char *argv[], void *userdata) {
 
 static int display_services(int argc, char *argv[], void *userdata) {
         _cleanup_(table_unrefp) Table *t = NULL;
-        _cleanup_(closedirp) DIR *d = NULL;
+        _cleanup_closedir_ DIR *d = NULL;
         int r;
 
         d = opendir("/run/systemd/userdb/");
@@ -952,7 +931,7 @@ static int display_services(int argc, char *argv[], void *userdata) {
 
         FOREACH_DIRENT(de, d, return -errno) {
                 _cleanup_free_ char *j = NULL, *no = NULL;
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 j = path_join("/run/systemd/userdb/", de->d_name);
                 if (!j)
