@@ -9,6 +9,7 @@
 #include "conf-parser.h"
 #include "escape.h"
 #include "fd-util.h"
+#include "fileio.h"
 #include "generator.h"
 #include "log.h"
 #include "nulstr-util.h"
@@ -516,8 +517,9 @@ int xdg_autostart_service_generate_unit(
                 const XdgAutostartService *service,
                 const char *dest) {
 
-        _cleanup_free_ char *path_escaped = NULL, *exec_start = NULL, *unit = NULL;
+        _cleanup_free_ char *path_escaped = NULL, *exec_start = NULL;
         _cleanup_fclose_ FILE *f = NULL;
+        _cleanup_strv_free_ char **only_show_in = NULL, **not_show_in = NULL;
         int r;
 
         assert(service);
@@ -544,6 +546,18 @@ int xdg_autostart_service_generate_unit(
                 return 0;
         }
 
+        if (service->only_show_in) {
+                only_show_in = strv_copy(service->only_show_in);
+                if (!only_show_in)
+                        return log_oom();
+        }
+
+        if (service->not_show_in) {
+                not_show_in = strv_copy(service->not_show_in);
+                if (!not_show_in)
+                        return log_oom();
+        }
+
         /* The TryExec key cannot be checked properly from the systemd unit, it is trivial to check using
          * find_executable though. */
         if (service->try_exec) {
@@ -558,30 +572,46 @@ int xdg_autostart_service_generate_unit(
 
         r = xdg_autostart_format_exec_start(service->exec_string, &exec_start);
         if (r < 0) {
-                log_warning_errno(r, "%s: not generating unit, error parsing Exec= line: %m", service->path);
+                log_full_errno(r == -ENOENT ? LOG_INFO : LOG_WARNING, r,
+                               r == -ENOENT ? "%s: not generating unit, executable specified in Exec= does not exist."
+                                            : "%s: not generating unit, error parsing Exec= line: %m",
+                               service->path);
                 return 0;
         }
 
         if (service->gnome_autostart_phase) {
-                /* There is no explicit value for the "Application" phase. */
-                log_debug("%s: not generating unit, startup phases are not supported.", service->path);
-                return 0;
+                /* There is no explicit value for the "Application" phase.
+                 *
+                 * On GNOME secondary startup mechanism handles desktop files with startup phases set.
+                 * We want to mark these as "NotShowIn=GNOME"
+                 *
+                 * If that means no-one will load them, we can get skip it entirely.
+                 */
+                if (strv_contains(only_show_in, "GNOME")) {
+                        strv_remove(only_show_in, "GNOME");
+
+                        if (strv_isempty(only_show_in)) {
+                                log_debug("%s: GNOME startup phases are handled separately. Skipping.",
+                                          service->path);
+                                return 0;
+                        }
+                }
+                log_debug("%s: GNOME startup phases are handled separately, marking as NotShowIn=GNOME.",
+                          service->path);
+
+                if (strv_extend(&not_show_in, "GNOME") < 0)
+                        return log_oom();
         }
 
         path_escaped = specifier_escape(service->path);
         if (!path_escaped)
                 return log_oom();
 
-        unit = path_join(dest, service->name);
-        if (!unit)
-                return log_oom();
-
-        f = fopen(unit, "wxe");
-        if (!f)
-                return log_error_errno(errno, "%s: failed to create unit file %s: %m", service->path, unit);
+        r = generator_open_unit_file(dest, /* source = */ NULL, service->name, &f);
+        if (r < 0)
+                return r;
 
         fprintf(f,
-                "# Automatically generated by systemd-xdg-autostart-generator\n\n"
                 "[Unit]\n"
                 "Documentation=man:systemd-xdg-autostart-generator(8)\n"
                 "SourcePath=%s\n"
@@ -608,7 +638,7 @@ int xdg_autostart_service_generate_unit(
                 "ExitType=cgroup\n"
                 "ExecStart=:%s\n"
                 "Restart=no\n"
-                "TimeoutSec=5s\n"
+                "TimeoutStopSec=5s\n"
                 "Slice=app.slice\n",
                 exec_start);
 
@@ -623,22 +653,22 @@ int xdg_autostart_service_generate_unit(
         }
 
         /* Generate an ExecCondition to check $XDG_CURRENT_DESKTOP */
-        if (!strv_isempty(service->only_show_in) || !strv_isempty(service->not_show_in)) {
-                _cleanup_free_ char *only_show_in = NULL, *not_show_in = NULL, *e_only_show_in = NULL, *e_not_show_in = NULL;
+        if (!strv_isempty(only_show_in) || !strv_isempty(not_show_in)) {
+                _cleanup_free_ char *only_show_in_string = NULL, *not_show_in_string = NULL, *e_only_show_in = NULL, *e_not_show_in = NULL;
 
-                only_show_in = strv_join(service->only_show_in, ":");
-                not_show_in = strv_join(service->not_show_in, ":");
-                if (!only_show_in || !not_show_in)
+                only_show_in_string = strv_join(only_show_in, ":");
+                not_show_in_string = strv_join(not_show_in, ":");
+                if (!only_show_in_string || !not_show_in_string)
                         return log_oom();
 
-                e_only_show_in = cescape(only_show_in);
-                e_not_show_in = cescape(not_show_in);
+                e_only_show_in = cescape(only_show_in_string);
+                e_not_show_in = cescape(not_show_in_string);
                 if (!e_only_show_in || !e_not_show_in)
                         return log_oom();
 
                 /* Just assume the values are reasonably sane */
                 fprintf(f,
-                        "ExecCondition=" ROOTLIBEXECDIR "/systemd-xdg-autostart-condition \"%s\" \"%s\"\n",
+                        "ExecCondition=" LIBEXECDIR "/systemd-xdg-autostart-condition \"%s\" \"%s\"\n",
                         e_only_show_in,
                         e_not_show_in);
         }
@@ -655,6 +685,10 @@ int xdg_autostart_service_generate_unit(
         if (r < 0)
                 return r;
 
+        r = fflush_and_check(f);
+        if (r < 0)
+                return log_error_errno(r, "Failed to write unit %s: %m", service->name);
+
         log_debug("%s: symlinking %s in xdg-desktop-autostart.target/.wants%s",
                   service->path, service->name, special_glyph(SPECIAL_GLYPH_ELLIPSIS));
         return generator_add_symlink(dest, "xdg-desktop-autostart.target", "wants", service->name);
