@@ -17,8 +17,11 @@ KERNEL_APPEND="foo -- -z bar --- baz $KERNEL_APPEND"
 . "${TEST_BASE_DIR:?}/test-functions"
 
 test_append_files() {
-    # install tests manually so the test is functional even when -Dinstall-tests=false
-    local dst="${1:?}/usr/lib/systemd/tests/testdata/units/"
+    local workspace="${1:?}"
+    local dst
+
+    # Install tests manually so the test is functional even when -Dinstall-tests=false
+    dst="$workspace/usr/lib/systemd/tests/testdata/units/"
     mkdir -p "$dst"
     cp -v "$TEST_UNITS_DIR"/{testsuite-01,end}.service "$TEST_UNITS_DIR/testsuite.target" "$dst"
 }
