@@ -12,19 +12,14 @@ TEST_INSTALL_VERITY_MINIMAL=1
 # shellcheck source=test/test-functions
 . "${TEST_BASE_DIR:?}/test-functions"
 
-# Need loop devices for mounting images
 test_append_files() {
-    (
-        instmods loop =block
-        instmods squashfs =squashfs
-        instmods dm_verity =md
-        instmods overlay =overlayfs
-        install_dmevent
-        generate_module_dependencies
-        inst_binary mksquashfs
-        inst_binary unsquashfs
-        install_verity_minimal
-    )
+    instmods squashfs =squashfs
+    instmods dm_verity =md
+    install_dmevent
+    generate_module_dependencies
+    inst_binary mksquashfs
+    inst_binary unsquashfs
+    install_verity_minimal
 }
 
 do_test "$@"
