@@ -4,6 +4,7 @@ set -e
 
 TEST_DESCRIPTION="test systemd-repart"
 IMAGE_NAME="repart"
+IMAGE_ADDITIONAL_ROOT_SIZE=1000
 TEST_FORCE_NEWIMAGE=1
 
 # shellcheck source=test/test-functions
@@ -15,11 +16,13 @@ test_append_files() {
         instmods dm_verity =md
         instmods erofs
         generate_module_dependencies
+        image_install veritysetup
         image_install -o mksquashfs
         image_install -o mkfs.erofs
     fi
 
     inst_binary mcopy
+    image_install mkswap
     if command -v openssl >/dev/null 2>&1; then
         inst_binary openssl
     fi
