@@ -16,7 +16,7 @@ systemd-run --on-clock-change touch /tmp/clock-changed
 test ! -f /tmp/timezone-changed
 test ! -f /tmp/clock-changed
 
-timedatectl set-timezone Europe/Kyiv
+timedatectl set-timezone Europe/Kiev
 
 while test ! -f /tmp/timezone-changed ; do sleep .5 ; done
 
@@ -26,6 +26,4 @@ while test ! -f /tmp/clock-changed ; do sleep .5 ; done
 
 systemd-analyze log-level info
 
-echo OK >/testok
-
-exit 0
+touch /testok
