@@ -239,11 +239,12 @@ testcase_vc_keymap() {
     assert_in "VC Keymap:" "$(localectl)"
 
     for i in $(localectl list-keymaps); do
+        # clear previous conversion from VC -> X11 keymap
+        systemctl stop systemd-localed.service
+        wait_vconsole_setup
+        rm -f /etc/vconsole.conf /etc/X11/xorg.conf.d/00-keyboard.conf /etc/default/keyboard
+
         # set VC keymap
-
-        # Skip lv keymap and friends, otherwise the sanitizer detects heap-buffer-overflow in libxkbcommon.
-        [[ "$i" =~ ^lv ]] && continue
-
         assert_rc 0 localectl set-keymap "$i"
         output=$(localectl)
 
